'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const index = require('./index-a6a78ee3.js');
const overlays = require('./overlays-33555855.js');
const helpers = require('./helpers-c4f9304b.js');
const toggleScrollingBody = require('./toggle-scrolling-body-5e1f7be7.js');

const attachComponent = async (delegate, container, component, cssClasses, componentProps) => {
  if (delegate) {
    return delegate.attachViewToDom(container, component, componentProps, cssClasses);
  }
  if (typeof component !== 'string' && !(component instanceof HTMLElement)) {
    throw new Error('framework delegate is missing');
  }
  const el = typeof component === 'string'
    ? container.ownerDocument && container.ownerDocument.createElement(component)
    : component;
  if (cssClasses) {
    cssClasses.forEach(c => el.classList.add(c));
  }
  if (componentProps) {
    Object.assign(el, componentProps);
  }
  container.appendChild(el);
  await new Promise(resolve => helpers.componentOnReady(el, resolve));
  return el;
};
const detachComponent = (delegate, element) => {
  if (element) {
    if (delegate) {
      const container = element.parentElement;
      return delegate.removeViewFromDom(container, element);
    }
    element.remove();
  }
  return Promise.resolve();
};

const getClassMap = (classes) => {
  const map = {};
  getClassList(classes).forEach(c => (map[c] = true));
  return map;
};
const getClassList = (classes) => {
  if (classes !== undefined) {
    const array = Array.isArray(classes) ? classes : classes.split(' ');
    return array
      .filter(c => c != null)
      .map(c => c.trim())
      .filter(c => c !== '');
  }
  return [];
};

const Modal = class {
  constructor(hostRef) {
    index.registerInstance(this, hostRef);
    this.didPresent = index.createEvent(this, "balModalDidPresent", 7);
    this.willPresent = index.createEvent(this, "balModalWillPresent", 7);
    this.willDismiss = index.createEvent(this, "balModalWillDismiss", 7);
    this.didDismiss = index.createEvent(this, "balModalDidDismiss", 7);
    this.bodyScrollBlocker = toggleScrollingBody.BodyScrollBlocker();
    this.presented = false;
    /**
     * Defines the width of the modal body
     */
    this.modalWidth = 640;
    /**
     * Defines the space/padding of the modal
     */
    this.space = '';
    /**
     * If `true`, a backdrop will be displayed behind the modal.
     */
    this.hasBackdrop = true;
    /**
     * If `true`, the modal can be closed with the escape key or the little close button.
     */
    this.isClosable = true;
  }
  connectedCallback() {
    overlays.prepareOverlay(this);
  }
  /**
   * @internal Opens the modal.
   */
  async open() {
    await helpers.deepReady(this.usersElement);
    this.setModalActiveOnBody();
    index.writeTask(() => {
      if (this.modalBackgroundElement) {
        this.modalBackgroundElement.classList.add('fadeIn');
      }
      if (this.modalContentElement) {
        this.modalContentElement.classList.add('fadeInDown');
      }
    });
    if (this.presented) {
      return;
    }
    this.willPresent.emit();
    this.presented = true;
    await helpers.wait(150);
    index.writeTask(() => {
      if (this.modalBackgroundElement) {
        this.modalBackgroundElement.classList.remove('fadeIn');
      }
      if (this.modalContentElement) {
        this.modalContentElement.classList.remove('fadeInDown');
      }
    });
    this.didPresent.emit();
  }
  /**
   * @internal Closes the modal.
   */
  async close() {
    this.unsetModalActiveOnBody();
    this.presented = false;
  }
  /**
   * Presents the modal through the modal controller
   */
  async present() {
    this.setModalActiveOnBody();
    if (this.presented) {
      return;
    }
    const container = this.el.querySelector(`.bal-modal__content`);
    if (!container) {
      throw new Error('container is undefined');
    }
    const componentProps = Object.assign(Object.assign({}, this.componentProps), { modal: this.el });
    this.usersElement = await attachComponent(this.delegate, container, this.component, [], componentProps);
    await this.open();
  }
  /**
   * Closes the presented modal with the modal controller
   */
  async dismiss(data, role) {
    this.unsetModalActiveOnBody();
    if (this.delegate === undefined) {
      await this.close();
      return true;
    }
    this.willDismiss.emit({ data, role });
    const dismissed = await overlays.dismiss(this, data, role, async () => {
      index.writeTask(() => {
        if (this.modalBackgroundElement) {
          this.modalBackgroundElement.classList.add('fadeOut');
        }
        if (this.modalContentElement) {
          this.modalContentElement.classList.add('fadeOutUp');
        }
      });
      await helpers.wait(140);
    });
    if (dismissed) {
      await detachComponent(this.delegate, this.usersElement);
    }
    return dismissed;
  }
  /**
   * Returns a promise that resolves when the modal did dismiss.
   */
  onDidDismiss() {
    return overlays.eventMethod(this.el, 'balModalDidDismiss');
  }
  /**
   * Returns a promise that resolves when the modal will dismiss.
   */
  onWillDismiss() {
    return overlays.eventMethod(this.el, 'balModalWillDismiss');
  }
  async onClickCloseButton(event) {
    if (this.isClosable && this.presented && event && event.target) {
      const element = event.target;
      const closestBalButton = element.closest('bal-button');
      if (closestBalButton && closestBalButton.hasAttribute('modal-close')) {
        await this.dismiss(undefined, 'model-close');
      }
    }
  }
  async handleKeyUp(event) {
    const modals = Array.from(document.querySelectorAll('bal-modal')).filter(el => el.hasAttribute('aria-presented'));
    const numbers = modals
      .map(el => el.overlayIndex)
      .map(num => parseInt(num))
      .filter(num => !Number.isNaN(num));
    const max = Math.max(...numbers);
    if (this.overlayIndex === max) {
      event.preventDefault();
      event.stopPropagation();
      if (this.presented && this.isClosable) {
        if (event.key === 'Escape' || event.key === 'Esc') {
          event.preventDefault();
          if (this.delegate) {
            await this.dismiss(undefined, 'model-escape');
          }
          else {
            await this.close();
          }
        }
      }
    }
  }
  setModalActiveOnBody() {
    this.bodyScrollBlocker.block();
  }
  unsetModalActiveOnBody() {
    this.bodyScrollBlocker.allow();
  }
  render() {
    return (index.h(index.Host, { "aria-modal": "true", "aria-presented": this.presented ? 'true' : null, "data-testid": this.dataTestId, tabindex: "-1", class: Object.assign({ 'bal-modal': true, 'bal-modal--is-closable': this.isClosable, 'bal-modal--is-active': this.presented }, getClassMap(this.cssClass)), style: {
        '--bal-width': `${this.modalWidth}px`,
      } }, index.h("div", { class: {
        'bal-modal__background': true,
        'is-hidden': !this.hasBackdrop,
      }, ref: div => (this.modalBackgroundElement = div) }), index.h("div", { class: {
        'bal-modal__container': true,
      }, ref: div => (this.modalContainerElement = div) }, index.h("div", { class: {
        'bal-modal__content': true,
        [`bal-modal__content--has-spacing-${this.space}`]: true,
      }, ref: div => (this.modalContentElement = div) }, index.h("slot", null)))));
  }
  get el() { return index.getElement(this); }
};

exports.bal_modal = Modal;
