import { Component, Host, h, Prop, Method, Event, Watch } from '@stencil/core';
import { debounceEvent } from '../../helpers/helpers';
export class Accordion {
  constructor() {
    this.didInit = false;
    /**
     * If `true` the accordion is open.
     */
    this.value = false;
    /**
     * Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`.
     */
    this.debounce = 0;
    /**
     * Label of the open trigger button
     */
    this.openLabel = '';
    /**
     * BalIcon of the open trigger button
     */
    this.openIcon = 'plus';
    /**
     * Label of the close trigger button
     */
    this.closeLabel = '';
    /**
     * BalIcon of the close trigger button
     */
    this.closeIcon = 'close';
    /**
     * If `true` the accordion is used on the bottom of a card
     */
    this.card = false;
  }
  async valueChanged(newValue, oldValue) {
    if (this.didInit && newValue !== oldValue) {
      this.balChange.emit(newValue);
    }
  }
  debounceChanged() {
    this.balChange = debounceEvent(this.balChange, this.debounce);
  }
  connectedCallback() {
    this.debounceChanged();
  }
  componentDidLoad() {
    this.didInit = true;
    if (this.value !== undefined) {
      this.valueChanged(this.value, false);
    }
  }
  /**
   * Opens the accordion
   */
  async present() {
    this.value = true;
  }
  /**
   * Closes the accordion
   */
  async dismiss() {
    this.value = false;
  }
  /**
   * Triggers the accordion
   */
  async toggle() {
    this.value = !this.value;
  }
  get label() {
    return this.value ? this.closeLabel : this.openLabel;
  }
  get icon() {
    return this.value ? this.closeIcon : this.openIcon;
  }
  render() {
    const label = this.value ? this.closeLabel : this.openLabel;
    const icon = this.value ? this.closeIcon : this.openIcon;
    return (h(Host, { class: {
        'accordion': true,
        'mt-5': this.card,
      }, "aria-presented": this.value ? 'true' : null },
      h("div", { class: {
          'is-flex is-flex-direction-column-reverse': true,
        } },
        h("div", { class: {
            'accordion-trigger': true,
            'px-5': this.card,
            'pb-5': this.card,
          } },
          h("bal-button", { class: {
              'data-test-accordion-trigger': true,
            }, expanded: true, color: 'info', icon: icon, onClick: () => this.toggle() }, label)),
        h("div", { class: {
            'accordion-content': true,
          }, style: {
            display: this.value ? 'block' : 'none',
          } },
          h("slot", null)))));
  }
  static get is() { return "bal-accordion"; }
  static get properties() { return {
    "value": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the accordion is open."
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "false"
    },
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`."
      },
      "attribute": "debounce",
      "reflect": false,
      "defaultValue": "0"
    },
    "openLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label of the open trigger button"
      },
      "attribute": "open-label",
      "reflect": false,
      "defaultValue": "''"
    },
    "openIcon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "BalIcon of the open trigger button"
      },
      "attribute": "open-icon",
      "reflect": false,
      "defaultValue": "'plus'"
    },
    "closeLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label of the close trigger button"
      },
      "attribute": "close-label",
      "reflect": false,
      "defaultValue": "''"
    },
    "closeIcon": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "BalIcon of the close trigger button"
      },
      "attribute": "close-icon",
      "reflect": false,
      "defaultValue": "'close'"
    },
    "card": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the accordion is used on the bottom of a card"
      },
      "attribute": "card",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the accordion has opened or closed"
      },
      "complexType": {
        "original": "Events.BalAccordionChangeDetail",
        "resolved": "boolean",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../types"
          }
        }
      }
    }]; }
  static get methods() { return {
    "present": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Opens the accordion",
        "tags": []
      }
    },
    "dismiss": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the accordion",
        "tags": []
      }
    },
    "toggle": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Triggers the accordion",
        "tags": []
      }
    }
  }; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "valueChanged"
    }, {
      "propName": "debounce",
      "methodName": "debounceChanged"
    }]; }
}
