import { Component, h, Host, Prop } from '@stencil/core';
export class Heading {
  constructor() {
    /**
     * The actual heading level used in the HTML markup.
     */
    this.level = 'h1';
    /**
     * If `true` the heading gets displayed slimmer.
     */
    this.subtitle = false;
    /**
     * Defines at which position the heading has spacing.
     */
    this.space = 'bottom';
    /**
     * The theme type of the toast. Given by bulma our css framework.
     */
    this.color = '';
    /**
     * If `true` the color gets inverted for dark backgrounds
     */
    this.inverted = false;
    /**
     * If `true` adds a text shadow to improve readability on image background
     * */
    this.shadow = false;
  }
  get fontSize() {
    const isHeading = (size) => size.startsWith('h');
    const parseDisplay = (size) => (size.startsWith('display') ? size : size);
    const parseSize = (size) => (isHeading(size) ? size.replace('h', '') : parseDisplay(size));
    const formatSize = (size) => `is-size-${parseSize(size)}`;
    return formatSize(this.visualLevel ? this.visualLevel : this.level);
  }
  get fontColor() {
    const parseColor = (color) => (color !== '' ? `has-text-${color}` : '');
    return parseColor(this.inverted ? 'white' : this.color);
  }
  margins(spacingLevel) {
    const spacing = [];
    if (this.space === 'top' || this.space === 'all') {
      spacing.push(`mt-${spacingLevel}`);
    }
    if (this.space === 'bottom' || this.space === 'all') {
      spacing.push(`mb-${spacingLevel}`);
    }
    return spacing.join(' ');
  }
  get spacing() {
    switch (this.level) {
      case 'display':
      case 'display-2':
        return this.margins(4);
      case 'h1':
        return this.margins(3);
      case 'h2':
      case 'h3':
      case 'h4':
      case 'h5':
        return this.margins(2);
      default:
        return '';
    }
  }
  render() {
    const Heading = this.level.startsWith('display') ? 'h1' : this.level;
    return (h(Host, { class: { [this.spacing]: true } },
      h(Heading, { class: {
          'data-test-heading': true,
          'title': this.subtitle === false,
          'subtitle': this.subtitle === true,
          'has-blur-shadow': this.shadow,
          [this.fontSize]: true,
          [this.fontColor]: true,
        } },
        h("slot", null))));
  }
  static get is() { return "bal-heading"; }
  static get properties() { return {
    "level": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalHeadingLevel",
        "resolved": "\"display\" | \"display-2\" | \"h1\" | \"h2\" | \"h3\" | \"h4\" | \"h5\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The actual heading level used in the HTML markup."
      },
      "attribute": "level",
      "reflect": false,
      "defaultValue": "'h1'"
    },
    "visualLevel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalHeadingLevel",
        "resolved": "\"display\" | \"display-2\" | \"h1\" | \"h2\" | \"h3\" | \"h4\" | \"h5\" | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Make the visual style mimic a specific heading level.\nThis option allows you to make e.g. h1 visually look like h3,\nbut still keep it h1 in the markup."
      },
      "attribute": "visual-level",
      "reflect": false
    },
    "subtitle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the heading gets displayed slimmer."
      },
      "attribute": "subtitle",
      "reflect": false,
      "defaultValue": "false"
    },
    "space": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'none' | 'bottom' | 'top' | 'all'",
        "resolved": "\"all\" | \"bottom\" | \"none\" | \"top\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines at which position the heading has spacing."
      },
      "attribute": "space",
      "reflect": false,
      "defaultValue": "'bottom'"
    },
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalHeadingColor",
        "resolved": "\"\" | \"blue\" | \"danger\" | \"info\" | \"primary\" | \"success\" | \"warning\" | \"white\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The theme type of the toast. Given by bulma our css framework."
      },
      "attribute": "color",
      "reflect": false,
      "defaultValue": "''"
    },
    "inverted": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the color gets inverted for dark backgrounds"
      },
      "attribute": "inverted",
      "reflect": false,
      "defaultValue": "false"
    },
    "shadow": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` adds a text shadow to improve readability on image background"
      },
      "attribute": "shadow",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
}
