import { Component, Host, h, Method, State, Prop, Element, Listen } from '@stencil/core';
import { attachComponentToConfig, detachComponentToConfig } from '../../config';
import { isPlatform } from '../../utils/platform';
import { BEM } from '../../utils/bem';
import { preventDefault } from '../form/bal-select/utils/utils';
import { BodyScrollBlocker } from '../../utils/toggle-scrolling-body';
export class Hint {
  constructor() {
    this.bodyScrollBlocker = BodyScrollBlocker();
    this.isActive = false;
    this.innerCloseLabel = 'Close';
    this.isMobile = isPlatform('mobile');
    /**
     * Disables the close button for tablet and desktop
     */
    this.small = false;
    this.onPopoverChange = (event) => {
      this.isActive = event.detail;
      preventDefault(event);
    };
  }
  async resizeHandler() {
    const isCurrentMobile = isPlatform('mobile');
    if (isCurrentMobile !== this.isMobile) {
      this.isActive = false;
    }
    this.isMobile = isCurrentMobile;
  }
  connectedCallback() {
    attachComponentToConfig(this);
  }
  disconnectedCallback() {
    detachComponentToConfig(this);
  }
  componentDidRender() {
    this.updateContent();
  }
  configChanged(state) {
    if (!this.closeLabel) {
      switch (state.language) {
        case 'de':
          this.innerCloseLabel = 'Schliessen';
          break;
        case 'fr':
          this.innerCloseLabel = 'Fermer';
          break;
        case 'it':
          this.innerCloseLabel = 'Chiudere';
          break;
        case 'nl':
          this.innerCloseLabel = 'Sluiten';
          break;
        default:
          this.innerCloseLabel = 'Close';
          break;
      }
    }
  }
  /**
   * Toggles the hint box.
   */
  async toggle() {
    if (this.isActive) {
      this.dismiss();
    }
    else {
      this.present();
    }
  }
  /**
   * Opens the hint box.
   */
  async present() {
    if (this.popoverElement) {
      this.popoverElement.present();
    }
    if (this.isMobile) {
      this.bodyScrollBlocker.block();
    }
    this.isActive = true;
  }
  /**
   * Closes the hint box.
   */
  async dismiss() {
    if (this.popoverElement) {
      this.popoverElement.dismiss();
    }
    if (this.isMobile) {
      this.bodyScrollBlocker.allow();
    }
    this.isActive = false;
  }
  updateContent() {
    if (this.hintContentEl && this.slotWrapperEl) {
      this.hintContentEl.innerHTML = this.slotWrapperEl.innerHTML;
    }
  }
  render() {
    const block = BEM.block('hint');
    const elIcon = block.element('icon');
    const elContent = block.element('content');
    const elButtons = elContent.element('buttons');
    const padding = this.isMobile ? 0 : 8;
    const offsetY = this.isMobile ? 0 : 16;
    const TriggerIcon = () => {
      return (h("bal-icon", { class: Object.assign(Object.assign({}, elIcon.class()), { 'data-test-hint-trigger': true }), "bal-popover-trigger": true, "aria-haspopup": "true", role: "button", name: "info-circle", onClick: () => this.toggle() }));
    };
    const HintContent = () => {
      return (h("div", { class: Object.assign(Object.assign({}, elContent.class()), { 'data-test-hint-content': true }) },
        h("div", { ref: el => (this.hintContentEl = el) }),
        h("bal-button-group", { class: Object.assign(Object.assign({}, elButtons.class()), elButtons.modifier('is-hidden-desktop').class(this.small)) },
          h("bal-button", { class: "data-test-hint-close", color: "info", onClick: () => this.dismiss() }, this.closeLabel ? this.closeLabel : this.innerCloseLabel))));
    };
    const MobileOverlay = () => {
      const elOverlay = block.element('overlay');
      return (h("div", { class: Object.assign({}, elOverlay.class()) },
        h(TriggerIcon, null),
        h("div", { class: Object.assign(Object.assign({}, elOverlay.element('content').class()), elOverlay.element('content').modifier('active').class(this.isActive)) },
          h(HintContent, null))));
    };
    const Popover = () => {
      return (h("div", { class: Object.assign({}, block.element('popover').class()) },
        h("bal-popover", { hint: true, position: "right", offsetX: 0, offsetY: offsetY, padding: padding, ref: el => (this.popoverElement = el), onBalChange: this.onPopoverChange },
          h(TriggerIcon, null),
          h("bal-popover-content", { color: "grey" },
            h(HintContent, null)))));
    };
    const HintElement = this.isMobile ? MobileOverlay : Popover;
    return (h(Host, { class: Object.assign({}, block.class()) },
      h(HintElement, null),
      h("div", { ref: el => (this.slotWrapperEl = el), style: { display: 'none' } },
        h("slot", null))));
  }
  static get is() { return "bal-hint"; }
  static get properties() { return {
    "closeLabel": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Text for the close button."
      },
      "attribute": "close-label",
      "reflect": false
    },
    "small": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Disables the close button for tablet and desktop"
      },
      "attribute": "small",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "isActive": {},
    "innerCloseLabel": {},
    "isMobile": {}
  }; }
  static get methods() { return {
    "toggle": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Toggles the hint box.",
        "tags": []
      }
    },
    "present": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Opens the hint box.",
        "tags": []
      }
    },
    "dismiss": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the hint box.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "element"; }
  static get listeners() { return [{
      "name": "resize",
      "method": "resizeHandler",
      "target": "window",
      "capture": false,
      "passive": true
    }]; }
}
