import { Component, Element, h, Host, Prop, State, Event } from '@stencil/core';
import { BEM } from '../../../utils/bem';
export class NavbarBrand {
  constructor() {
    this.isMenuActive = false;
    /**
     * Link target
     */
    this.linkTarget = '_blank';
    /**
     * @deprecated Use interface on bal-navbar instead.
     * If `true` the navbar does not have a mobil version. Only shows logo and an app title.
     */
    this.simple = false;
    /**
     * @internal
     * Defines the type of navbar. App is used for almost every web applications
     * like the portal app. For our sales funnel we recommend to use the simple navbar.
     * Meta and main are used for the website.
     */
    this.interface = 'app';
  }
  componentWillLoad() {
    if (window.matchMedia) {
      window.matchMedia('(min-width: 960px)').addEventListener('change', this.resetIsMenuActive.bind(this));
    }
  }
  async resetIsMenuActive(ev) {
    if (ev.matches && !this.simple) {
      this.toggle(false);
    }
  }
  async toggle(isMenuActive) {
    this.isMenuActive = isMenuActive;
    const navbar = this.el.closest('bal-navbar');
    if (navbar) {
      const navbarMenuElement = navbar.querySelector('bal-navbar-menu');
      if (navbarMenuElement && !this.simple) {
        await navbarMenuElement.toggle(this.isMenuActive);
      }
    }
  }
  async onClick() {
    this.toggle(!this.isMenuActive);
  }
  render() {
    const navbarBrandEl = BEM.block('navbar').element('brand');
    const logoTemplate = this.logo ? (h("img", { class: Object.assign({}, navbarBrandEl.element('logo').class()), src: this.logo, alt: "" })) : (h("bal-logo", { animated: true, color: 'white' }));
    return (h(Host, { class: Object.assign(Object.assign({}, navbarBrandEl.class()), navbarBrandEl.modifier(`context-${this.interface}`).class()) },
      this.href ? (h("a", { href: this.href, target: this.linkTarget, onClick: (event) => this.balNavigate.emit(event) }, logoTemplate)) : (logoTemplate),
      h("span", { class: Object.assign({}, navbarBrandEl.element('title').class()) },
        h("slot", null)),
      h("bal-button", { class: Object.assign(Object.assign({}, navbarBrandEl.element('burger').class()), { 'is-hidden': this.interface === 'simple' }), color: "light", inverted: true, square: true, icon: this.isMenuActive ? 'close' : 'menu-bars', onClick: () => this.onClick() })));
  }
  static get is() { return "bal-navbar-brand"; }
  static get properties() { return {
    "href": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Link of the logo / title."
      },
      "attribute": "href",
      "reflect": false
    },
    "linkTarget": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Link target"
      },
      "attribute": "link-target",
      "reflect": false,
      "defaultValue": "'_blank'"
    },
    "simple": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Use interface on bal-navbar instead.\nIf `true` the navbar does not have a mobil version. Only shows logo and an app title."
          }],
        "text": ""
      },
      "attribute": "simple",
      "reflect": false,
      "defaultValue": "false"
    },
    "logo": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Src to display a logo -> replaces the default Baloise Logo"
      },
      "attribute": "logo",
      "reflect": false
    },
    "interface": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalNavbarInterface",
        "resolved": "\"app\" | \"simple\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": "Defines the type of navbar. App is used for almost every web applications\nlike the portal app. For our sales funnel we recommend to use the simple navbar.\nMeta and main are used for the website."
          }],
        "text": ""
      },
      "attribute": "interface",
      "reflect": false,
      "defaultValue": "'app'"
    }
  }; }
  static get states() { return {
    "isMenuActive": {}
  }; }
  static get events() { return [{
      "method": "balNavigate",
      "name": "balNavigate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the link element has clicked"
      },
      "complexType": {
        "original": "MouseEvent",
        "resolved": "MouseEvent",
        "references": {
          "MouseEvent": {
            "location": "global"
          }
        }
      }
    }]; }
  static get elementRef() { return "el"; }
}
