import { Component, h, Host, Listen, Method, Prop, Watch, Element, Event, State } from '@stencil/core';
import { createPopper } from '@popperjs/core';
import { BEM } from '../../utils/bem';
import { isBrowser } from '../../utils/browser';
import { isPlatform } from '../../utils/platform';
export class Popover {
  constructor() {
    this.popoverId = `bal-po-${PopoverIds++}`;
    this.isTouch = isPlatform('touch');
    this.isInMainNav = false;
    this.backdropHeight = 0;
    /**
     * If `true` the popover has max-width on tablet and desktop. On mobile it uses the whole viewport.
     */
    this.hint = false;
    /**
     * If `true` the popover shows on hover
     */
    this.hover = false;
    /**
     * If `true` a little arrow is added, which points to the trigger element
     */
    this.arrow = false;
    /**
     * If `true` a backdrop is added
     */
    this.backdrop = false;
    /**
     * If `true` the popover is shown as a tooltip
     */
    this.tooltip = false;
    /**
     * Define the offset of the popover content.
     */
    this.offsetX = 0;
    /**
     * Define the offset of the popover content.
     */
    this.offsetY = 0;
    /**
     * Define padding of the overflow
     */
    this.padding = 0;
    /**
     * Define the position of the popover content.
     */
    this.position = 'bottom-start';
    /**
     * If `true` the popover content is open.
     */
    this.value = false;
    /**
     * If `true` there will be no backdrop
     */
    this.mobileTop = false;
  }
  /**
   * Update the native input element when the value changes
   */
  async valueChanged(newValue, oldValue) {
    if (newValue === true && newValue !== oldValue) {
      this.present({ force: true });
    }
    else {
      this.dismiss({ force: true });
    }
  }
  handlePopoverPrepare(event) {
    const popoverId = event.detail;
    if (this.popoverId !== popoverId) {
      this.dismiss();
    }
  }
  async clickOnOutside(event) {
    var _a;
    if (this.value) {
      if (!this.element.contains(event.target)) {
        this.value = false;
      }
      if ((_a = this.backdropElement) === null || _a === void 0 ? void 0 : _a.isEqualNode(event.target)) {
        this.value = false;
      }
    }
  }
  handleKeyUp(event) {
    if (this.value && (event.key === 'Escape' || event.key === 'Esc')) {
      event.preventDefault();
      this.dismiss();
    }
  }
  async tabOutside(event) {
    if (event.key === 'Tab' && !this.element.contains(document.activeElement) && this.value) {
      await this.toggle();
    }
  }
  async resizeHandler() {
    this.isTouch = isPlatform('touch');
    this.isInMainNav = this.footMobileNav !== null;
    this.backdropHeight = this.getBackdropHeight();
  }
  componentDidRender() {
    if (this.popperInstance) {
      this.popperInstance.setOptions((options) => (Object.assign(Object.assign({}, options), { placement: this.tooltip ? 'bottom' : this.position, modifiers: [
          ...options.modifiers.filter((m) => m.name !== 'offset' && m.name !== 'preventOverflow'),
          this.modifierOffset,
          this.modifierPreventOverflow,
        ] })));
      this.updatePopper();
    }
    // Bug fix for https://github.com/baloise-incubator/design-system/issues/551
    if (isBrowser('Safari') && !this.isTouch) {
      clearTimeout(this.componentDidRenderTimer);
      this.componentDidRenderTimer = setTimeout(() => {
        var _a;
        const triggerWidth = (_a = this.element) === null || _a === void 0 ? void 0 : _a.clientWidth;
        if (triggerWidth) {
          this.element.style.maxWidth = `${triggerWidth}px`;
        }
        else {
          this.element.style.maxWidth = `initial`;
        }
      });
    }
  }
  get footMobileNav() {
    return this.element.closest('[slot="meta-mobile-foot"]');
  }
  componentDidLoad() {
    this.isInMainNav = this.footMobileNav !== null;
    this.isTouch = isPlatform('touch');
    this.backdropHeight = this.getBackdropHeight();
    if (this.triggerElement && this.menuElement) {
      this.popperInstance = createPopper(this.triggerElement, this.menuElement, {
        placement: this.tooltip ? 'bottom' : this.position,
        modifiers: [this.modifierOffset, this.modifierPreventOverflow],
      });
      let showEvents = [];
      let hideEvents = [];
      if (this.tooltip) {
        showEvents = ['mouseenter', 'focus'];
        hideEvents = ['mouseleave', 'blur'];
      }
      showEvents.forEach(event => {
        if (this.triggerElement) {
          this.triggerElement.addEventListener(event, () => this.present());
        }
      });
      hideEvents.forEach(event => {
        if (this.triggerElement) {
          this.triggerElement.addEventListener(event, () => this.dismiss());
        }
      });
    }
  }
  /**
   * Open the popover
   */
  async present(options = { force: false }) {
    var _a, _b;
    if (!this.value || options.force) {
      (_a = this.menuElement) === null || _a === void 0 ? void 0 : _a.setAttribute('data-show', '');
      (_b = this.menuElement) === null || _b === void 0 ? void 0 : _b.setAttribute('aria-hidden', 'false');
      if (this.menuInnerElement) {
        this.menuInnerElement.scrollTo(0, 0);
      }
      this.balPopoverPrepare.emit(this.popoverId);
      this.value = true;
      this.popperInstance.setOptions((options) => (Object.assign(Object.assign({}, options), { modifiers: [...options.modifiers, { name: 'eventListeners', enabled: true }] })));
      this.updatePopper();
      this.balChange.emit(this.value);
    }
  }
  /**
   * Closes the popover
   */
  async dismiss(options = { force: false }) {
    var _a, _b;
    if (this.value || options.force) {
      (_a = this.menuElement) === null || _a === void 0 ? void 0 : _a.removeAttribute('data-show');
      (_b = this.menuElement) === null || _b === void 0 ? void 0 : _b.setAttribute('aria-hidden', 'true');
      this.value = false;
      this.popperInstance.setOptions((options) => (Object.assign(Object.assign({}, options), { modifiers: [...options.modifiers, { name: 'eventListeners', enabled: false }] })));
      this.updatePopper();
      this.balChange.emit(this.value);
    }
  }
  /**
   * Open or closes the popover
   */
  async toggle(options = { force: false }) {
    if (this.value) {
      await this.dismiss(options);
    }
    else {
      await this.present(options);
    }
  }
  updatePopper() {
    this.popperInstance.update();
  }
  get modifierOffset() {
    return {
      name: 'offset',
      options: {
        offset: [this.offsetX, this.offsetY + (this.tooltip || this.arrow ? 8 : 0)],
      },
    };
  }
  get modifierPreventOverflow() {
    return {
      name: 'preventOverflow',
      options: {
        padding: this.arrow || this.tooltip ? 8 : this.padding,
        altAxis: true,
      },
    };
  }
  get triggerElement() {
    return this.element.querySelector('[bal-popover-trigger]');
  }
  get menuElement() {
    return this.element.querySelector('bal-popover-content');
  }
  get menuInnerElement() {
    return this.element.querySelector('.bal-popover__content__inner');
  }
  getBackdropHeight() {
    return this.isInMainNav ? (window.innerHeight - (this.isTouch ? 64 : 48)) / 16 : window.innerHeight / 16;
  }
  render() {
    const block = BEM.block('popover');
    return (h(Host, { "aria-presented": this.value ? 'true' : null, "data-id": this.popoverId, class: Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, block.class()), block.modifier('active').class(this.value)), block.modifier('tooltip').class(this.tooltip)), block.modifier('arrow').class(this.arrow)), block.modifier('hint').class(this.hint)), block.modifier('backdrop').class(this.backdrop)) },
      h("slot", null),
      !this.mobileTop && (h("div", { ref: el => {
          this.backdropElement = el;
        }, class: Object.assign({}, block.element('backdrop').class(this.backdrop && this.value)), style: {
          '--bal-popover-backdrop-height': `${this.backdropHeight}rem`,
        } }))));
  }
  static get is() { return "bal-popover"; }
  static get properties() { return {
    "hint": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the popover has max-width on tablet and desktop. On mobile it uses the whole viewport."
      },
      "attribute": "hint",
      "reflect": false,
      "defaultValue": "false"
    },
    "hover": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the popover shows on hover"
      },
      "attribute": "hover",
      "reflect": false,
      "defaultValue": "false"
    },
    "arrow": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` a little arrow is added, which points to the trigger element"
      },
      "attribute": "arrow",
      "reflect": false,
      "defaultValue": "false"
    },
    "backdrop": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` a backdrop is added"
      },
      "attribute": "backdrop",
      "reflect": false,
      "defaultValue": "false"
    },
    "tooltip": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the popover is shown as a tooltip"
      },
      "attribute": "tooltip",
      "reflect": false,
      "defaultValue": "false"
    },
    "offsetX": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Define the offset of the popover content."
      },
      "attribute": "offset-x",
      "reflect": false,
      "defaultValue": "0"
    },
    "offsetY": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Define the offset of the popover content."
      },
      "attribute": "offset-y",
      "reflect": false,
      "defaultValue": "0"
    },
    "padding": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Define padding of the overflow"
      },
      "attribute": "padding",
      "reflect": false,
      "defaultValue": "0"
    },
    "position": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalPopoverPlacement",
        "resolved": "\"auto\" | \"auto-end\" | \"auto-start\" | \"bottom\" | \"bottom-end\" | \"bottom-start\" | \"left\" | \"left-end\" | \"left-start\" | \"right\" | \"right-end\" | \"right-start\" | \"top\" | \"top-end\" | \"top-start\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Define the position of the popover content."
      },
      "attribute": "position",
      "reflect": false,
      "defaultValue": "'bottom-start'"
    },
    "value": {
      "type": "boolean",
      "mutable": true,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the popover content is open."
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "false"
    },
    "mobileTop": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` there will be no backdrop"
      },
      "attribute": "mobile-top",
      "reflect": false,
      "defaultValue": "false"
    }
  }; }
  static get states() { return {
    "isTouch": {},
    "isInMainNav": {},
    "backdropHeight": {}
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Listen when the popover opens or closes. Returns the current value."
      },
      "complexType": {
        "original": "Events.BalPopoverChangeDetail",
        "resolved": "boolean",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../types"
          }
        }
      }
    }, {
      "method": "balPopoverPrepare",
      "name": "balPopoverPrepare",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": "- Use this to close unused popovers."
          }],
        "text": ""
      },
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "present": {
      "complexType": {
        "signature": "(options?: PopoverPresentOptions) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "PopoverPresentOptions": {
            "location": "local"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Open the popover",
        "tags": []
      }
    },
    "dismiss": {
      "complexType": {
        "signature": "(options?: PopoverPresentOptions) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "PopoverPresentOptions": {
            "location": "local"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the popover",
        "tags": []
      }
    },
    "toggle": {
      "complexType": {
        "signature": "(options?: PopoverPresentOptions) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "PopoverPresentOptions": {
            "location": "local"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Open or closes the popover",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "element"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "valueChanged"
    }]; }
  static get listeners() { return [{
      "name": "balPopoverPrepare",
      "method": "handlePopoverPrepare",
      "target": "body",
      "capture": false,
      "passive": false
    }, {
      "name": "click",
      "method": "clickOnOutside",
      "target": "document",
      "capture": false,
      "passive": false
    }, {
      "name": "keydown",
      "method": "handleKeyUp",
      "target": "window",
      "capture": false,
      "passive": false
    }, {
      "name": "keyup",
      "method": "tabOutside",
      "target": "document",
      "capture": false,
      "passive": false
    }, {
      "name": "resize",
      "method": "resizeHandler",
      "target": "window",
      "capture": false,
      "passive": true
    }]; }
}
let PopoverIds = 0;
