import { Component, h, Host, Element, State, Listen } from '@stencil/core';
import { BEM } from '../../utils/bem';
import { observeHasClassActive, observeItems } from '../../utils/observer';
import { isPlatform } from '../../utils/platform';
export class ProductSlider {
  constructor() {
    this.productWidth = 176;
    this.steps = 2;
    this.xPosition = 0;
    this.slideIndex = 0;
    this.lastSlide = 0;
    this.sliderLength = 0;
    /**
     * Set the slide to switch to
     * @param {number} slide :Set to switch to.
     */
    this.setSlide = (slide) => {
      const productContainer = this.getProductContainer();
      if (productContainer && slide >= 0 && slide <= this.lastSlide + 1) {
        this.slideIndex = slide > this.lastSlide ? this.lastSlide : slide;
        productContainer.style.transitionDuration = '1.2s';
        productContainer.style.transitionTimingFunction = 'cubic-bezier(0.23, 0.93, 0.13, 1)';
        productContainer.style.transform = `translate(-${this.slideIndex * this.productWidth}px)`;
      }
    };
  }
  async resizeHandler() {
    this.calculateLastSlide();
    this.setSlide(0);
  }
  touchStart(event) {
    const productContainer = this.getProductContainer();
    if (productContainer === null || productContainer === void 0 ? void 0 : productContainer.contains(event.target)) {
      this.xPosition = event.touches[0].pageX;
    }
  }
  touchEnd(event) {
    const productContainer = this.getProductContainer();
    if (productContainer === null || productContainer === void 0 ? void 0 : productContainer.contains(event.target)) {
      if (event.changedTouches[0].pageX < this.xPosition) {
        this.setSlide(this.slideIndex + this.steps);
      }
      else {
        this.setSlide(this.slideIndex - this.steps);
      }
    }
  }
  connectedCallback() {
    this.mutationO = observeItems(this.el, 'bal-product-slider-item', () => this.updateItems());
    const parentTabItem = this.el.closest('bal-tab-item');
    if (parentTabItem) {
      this.mutationTabO = observeHasClassActive(parentTabItem, () => this.calculateLastSlide());
    }
    this.updateItems();
  }
  disconnectedCallback() {
    if (this.mutationO) {
      this.mutationO.disconnect();
      this.mutationO = undefined;
    }
    if (this.mutationTabO) {
      this.mutationTabO.disconnect();
      this.mutationTabO = undefined;
    }
  }
  getChildItems() {
    return Array.from(this.el.querySelectorAll('bal-product-slider-item'));
  }
  getProductContainer() {
    return this.el.querySelector('.bal-product-slider__container__products');
  }
  updateItems() {
    this.items = this.getChildItems();
  }
  getParentWidth(element) {
    const parentWidth = element.parentNode.offsetWidth;
    if (parentWidth === 0) {
      return this.getParentWidth(element.parentNode.parentNode);
    }
    return parentWidth;
  }
  getOffsetWidth() {
    const elementOffsetWidth = this.el.offsetWidth;
    if (elementOffsetWidth === 0) {
      return this.getParentWidth(this.el.parentNode);
    }
    return elementOffsetWidth;
  }
  calculateLastSlide() {
    this.lastSlide = Math.ceil(this.sliderLength || this.items.length - this.getOffsetWidth() / this.productWidth);
  }
  render() {
    const block = BEM.block('product-slider');
    const container = block.element('container');
    const containerProducts = container.element('products');
    const controls = block.element('controls');
    const controlButton = controls.element('button');
    this.steps = isPlatform('mobile') ? 1 : 2;
    this.calculateLastSlide();
    const leftControlIsDisabled = this.slideIndex <= 0;
    const rightControlIsDisabled = this.slideIndex >= this.lastSlide;
    return (h(Host, { class: Object.assign({}, block.class()) },
      h("div", { class: Object.assign({}, container.class()) },
        h("div", { class: Object.assign({}, containerProducts.class()) },
          h("slot", null))),
      h("div", { class: Object.assign(Object.assign({}, controls.class()), { 'is-hidden': leftControlIsDisabled && rightControlIsDisabled }) },
        h("div", { class: Object.assign(Object.assign({}, controlButton.class()), controlButton.modifier('left').class()) },
          h("bal-button", { disabled: leftControlIsDisabled, onClick: () => this.setSlide(this.slideIndex > 1 ? this.slideIndex - this.steps : 0), color: "primary", square: true, rounded: true, icon: "caret-left" })),
        h("div", { class: Object.assign(Object.assign({}, controlButton.class()), controlButton.modifier('right').class()) },
          h("bal-button", { disabled: rightControlIsDisabled, onClick: () => this.setSlide(this.slideIndex + this.steps), color: "primary", square: true, rounded: true, icon: "caret-right" })))));
  }
  static get is() { return "bal-product-slider"; }
  static get states() { return {
    "items": {},
    "slideIndex": {},
    "lastSlide": {},
    "sliderLength": {}
  }; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "resize",
      "method": "resizeHandler",
      "target": "window",
      "capture": false,
      "passive": true
    }, {
      "name": "touchstart",
      "method": "touchStart",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "touchend",
      "method": "touchEnd",
      "target": undefined,
      "capture": false,
      "passive": true
    }]; }
}
