import { Component, Host, h, Element, State, Event, Method, Prop, Watch, Listen } from '@stencil/core';
import { debounceEvent } from '../../helpers/helpers';
import { watchForTabs } from './utils/watch-tabs';
import { TabList } from './components/tabs';
import { StepList } from './components/steps';
import { BEM } from '../../utils/bem';
import { getPlatforms, isPlatform } from '../../utils/platform';
import { stopEventBubbling } from '../../helpers/form-input.helpers';
export class Tabs {
  constructor() {
    this.didInit = false;
    this.tabsOptions = [];
    this.lineWidth = 0;
    this.lineOffsetLeft = 0;
    this.lineHeight = 0;
    this.lineOffsetTop = 0;
    this.isReady = false;
    this.platform = ['mobile'];
    /**
     * Defines the layout of the tabs.
     */
    this.interface = 'tabs';
    /**
     * Defines the layout of the tabs.
     */
    this.iconPosition = 'horizontal';
    /**
     * Defines the layout of the tabs. Right only works from the breakpoint
     * high-definition and beyond.
     */
    this.float = 'left';
    /**
     * If `true` the field expands over the whole width.
     */
    this.expanded = false;
    /**
     * If `true` the tabs container does not have a padding left or right.
     */
    this.spaceless = false;
    /**
     * If `true` the field expands over the whole width.
     */
    this.inverted = false;
    /**
     * If `true` the tabs is a block element and uses 100% of the width
     */
    this.fullwidth = false;
    /**
     * If `true` the tabs or steps can be clicked.
     */
    this.clickable = true;
    /**
     * If `true` a light border is shown for the tabs.
     */
    this.border = false;
    /**
     * Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`.
     */
    this.debounce = 0;
    /**
     * If `true` tabs are align vertically.
     */
    this.vertical = false;
    /**
     * The col size of the tabs on vertical mode.
     */
    this.verticalColSize = 'one-third';
    /**
     * If `true` the tabs are shown as a select component on mobile
     */
    this.selectOnMobile = false;
    this.value = undefined;
  }
  debounceChanged() {
    this.balChange = debounceEvent(this.balChange, this.debounce);
  }
  async valueChanged(newValue, oldValue) {
    this.tabs.forEach(t => t.setActive(t.value === this.value));
    if (this.didInit && newValue !== oldValue) {
      this.isReady = true;
    }
  }
  async resizeHandler() {
    this.platform = getPlatforms();
    this.moveLine(this.getTargetElement(this.value));
  }
  async popoverHandler() {
    this.platform = getPlatforms();
    this.moveLine(this.getTargetElement(this.value));
  }
  connectedCallback() {
    this.platform = getPlatforms();
    this.debounceChanged();
    this.updateTabs();
    this.mutationO = watchForTabs(this.el, 'bal-tab-item', () => {
      this.updateTabs();
    });
  }
  disconnectedCallback() {
    if (this.mutationO) {
      this.mutationO.disconnect();
      this.mutationO = undefined;
    }
  }
  componentDidLoad() {
    this.didInit = true;
    let value = this.value;
    if ((value === undefined || value === '') && this.interface !== 'navigation') {
      const availableTabs = this.tabsOptions.filter(t => !t.disabled);
      if (availableTabs.length > 0) {
        value = availableTabs[0].value;
      }
    }
    this.value = value;
    this.valueChanged(value, this.value);
  }
  componentDidRender() {
    this.moveLine(this.getTargetElement(this.value));
  }
  /**
   * Go to tab with the given value
   */
  async select(tab) {
    this.value = tab.value;
  }
  /**
   * @internal
   * Rerenders the line to mark the active tab.
   */
  async renderLine() {
    this.moveLine(this.getTargetElement(this.value), 100);
  }
  get tabs() {
    return Array.from(this.el.querySelectorAll('bal-tab-item'));
  }
  async updateTabs() {
    try {
      await Promise.all(this.tabs.map(value => value.getOptions())).then(tabsOptions => {
        this.tabsOptions = tabsOptions;
      });
      const activeTabs = this.tabsOptions.filter(t => t.active);
      if (activeTabs.length > 0) {
        const firstActiveTab = activeTabs[0];
        this.value = firstActiveTab.value;
      }
    }
    catch (e) {
      console.warn('[WARN] - Could not read tab options');
    }
  }
  async onSelectTab(event, tab) {
    if (tab.prevent || tab.disabled || !this.clickable) {
      stopEventBubbling(event);
    }
    if (!tab.disabled) {
      tab.navigate.emit(event);
      if (this.clickable) {
        let value = tab.value;
        if (this.interface === 'navigation' && value === this.value) {
          value = '';
        }
        if (value !== this.value) {
          this.balChange.emit(value);
          await this.select(tab);
        }
      }
    }
  }
  parseVertical() {
    if (this.vertical === 'true' || this.vertical === '') {
      return true;
    }
    if (this.vertical === 'false' || this.vertical === undefined) {
      return false;
    }
    return this.vertical;
  }
  moveLine(element, timeout = 0) {
    if (this.timeoutTimer) {
      clearTimeout(this.timeoutTimer);
    }
    this.timeoutTimer = setTimeout(() => {
      if (this.interface !== 'steps' && this.interface !== 'o-steps') {
        if (element) {
          let paddingLeft = 0;
          let paddingRight = 0;
          const listElement = element.closest('li');
          if (listElement) {
            paddingLeft = parseInt(window
              .getComputedStyle(listElement.firstChild)
              .getPropertyValue('padding-left')
              .slice(0, -2), 10);
            paddingRight = parseInt(window
              .getComputedStyle(listElement.firstChild)
              .getPropertyValue('padding-right')
              .slice(0, -2), 10);
          }
          const isMobile = isPlatform('mobile');
          const isTablet = isPlatform('tablet');
          const isVertical = this.parseVertical() === true;
          const isNavbarTablet = this.interface === 'navbar' && (isMobile || isTablet);
          const isVerticalMobile = isMobile && (this.vertical === 'mobile' || this.vertical === 'tablet');
          const isVerticalTablet = (isMobile || isTablet) && this.vertical === 'tablet';
          if (isVertical || isVerticalMobile || isVerticalTablet || isNavbarTablet) {
            if ((listElement === null || listElement === void 0 ? void 0 : listElement.clientHeight) !== undefined) {
              this.lineHeight = listElement.clientHeight - 8;
            }
            if ((listElement === null || listElement === void 0 ? void 0 : listElement.offsetTop) !== undefined) {
              this.lineOffsetTop = listElement.offsetTop + 4;
            }
          }
          else {
            if ((listElement === null || listElement === void 0 ? void 0 : listElement.clientWidth) !== undefined) {
              this.lineWidth = listElement.clientWidth - (this.expanded ? 0 : paddingLeft + paddingRight);
            }
            if ((listElement === null || listElement === void 0 ? void 0 : listElement.offsetLeft) !== undefined) {
              this.lineOffsetLeft = listElement.offsetLeft + (this.expanded ? 0 : paddingLeft);
            }
          }
        }
        else {
          this.lineWidth = 0;
        }
      }
    }, timeout);
  }
  getTargetElement(value) {
    const elements = Array.from(this.el.querySelectorAll('.data-test-tab-item'));
    return elements.filter(element => element.getAttribute('data-value') == value)[0];
  }
  isTabActive(tab) {
    return tab.value === this.value;
  }
  render() {
    const block = BEM.block('tabs');
    const isSteps = this.interface === 'steps' || this.interface === 'o-steps';
    const Tabs = isSteps ? StepList : TabList;
    const isMobile = isPlatform('mobile');
    const isTablet = isPlatform('tablet');
    const isPropVertical = this.parseVertical() === true;
    const isVerticalMobile = isMobile && (this.vertical === 'mobile' || this.vertical === 'tablet');
    const isVerticalTablet = (isMobile || isTablet) && this.vertical === 'tablet';
    const isVertical = isPropVertical || isVerticalMobile || isVerticalTablet;
    return (h(Host, { class: Object.assign(Object.assign(Object.assign(Object.assign({}, block.class()), block.modifier(`context-${this.interface}`).class()), block.modifier('vertical').class(this.parseVertical() === true)), block.modifier('fullwidth').class(this.expanded || this.fullwidth || isSteps)), "data-value": this.tabsOptions
        .filter(t => this.isTabActive(t))
        .map(t => t.value)
        .join(','), "data-label": this.tabsOptions
        .filter(t => this.isTabActive(t))
        .map(t => t.label)
        .join(',') },
      h("div", { class: {
          'columns is-multiline': this.interface !== 'meta' && this.interface !== 'navigation',
        } },
        h("div", { class: {
            'column': this.interface !== 'meta' && this.interface !== 'navigation',
            'is-full': !isVertical,
            [`is-${this.verticalColSize}`]: isVertical,
            'bal-tabs__col-items': true,
            'bal-tabs__col-items--vertical': isVertical,
          } },
          h(Tabs, { value: this.value, context: this.interface, inverted: this.inverted, spaceless: this.spaceless, tabs: this.tabsOptions, border: this.border, float: this.float, expanded: this.expanded, clickable: this.clickable, isReady: this.isReady, iconPosition: this.iconPosition, onSelectTab: (e, t) => this.onSelectTab(e, t), lineWidth: this.lineWidth, lineOffsetLeft: this.lineOffsetLeft, lineHeight: this.lineHeight, lineOffsetTop: this.lineOffsetTop, vertical: this.interface === 'navbar' ? 'tablet' : this.parseVertical(), selectOnMobile: this.selectOnMobile })),
        h("div", { class: {
            'column': this.interface !== 'meta' && this.interface !== 'navigation',
            'is-full': !isVertical,
            'bal-tabs__col-content': true,
            'bal-tabs__col-content--vertical': isVertical,
            'bal-tabs__col-content--full': this.verticalColSize === 'full',
          } },
          h("slot", null)))));
  }
  static get is() { return "bal-tabs"; }
  static get properties() { return {
    "interface": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalTabsInterface",
        "resolved": "\"meta\" | \"navbar\" | \"navigation\" | \"o-steps\" | \"steps\" | \"tabs\" | \"tabs-sub\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the layout of the tabs."
      },
      "attribute": "interface",
      "reflect": false,
      "defaultValue": "'tabs'"
    },
    "iconPosition": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalTabsIconPosition",
        "resolved": "\"horizontal\" | \"vertical\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the layout of the tabs."
      },
      "attribute": "icon-position",
      "reflect": false,
      "defaultValue": "'horizontal'"
    },
    "float": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalTabsFloat",
        "resolved": "\"left\" | \"right\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the layout of the tabs. Right only works from the breakpoint\nhigh-definition and beyond."
      },
      "attribute": "float",
      "reflect": false,
      "defaultValue": "'left'"
    },
    "expanded": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the field expands over the whole width."
      },
      "attribute": "expanded",
      "reflect": false,
      "defaultValue": "false"
    },
    "spaceless": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the tabs container does not have a padding left or right."
      },
      "attribute": "spaceless",
      "reflect": false,
      "defaultValue": "false"
    },
    "inverted": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the field expands over the whole width."
      },
      "attribute": "inverted",
      "reflect": false,
      "defaultValue": "false"
    },
    "fullwidth": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the tabs is a block element and uses 100% of the width"
      },
      "attribute": "fullwidth",
      "reflect": false,
      "defaultValue": "false"
    },
    "clickable": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the tabs or steps can be clicked."
      },
      "attribute": "clickable",
      "reflect": false,
      "defaultValue": "true"
    },
    "border": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` a light border is shown for the tabs."
      },
      "attribute": "border",
      "reflect": false,
      "defaultValue": "false"
    },
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`."
      },
      "attribute": "debounce",
      "reflect": false,
      "defaultValue": "0"
    },
    "vertical": {
      "type": "any",
      "mutable": false,
      "complexType": {
        "original": "Props.BalTabsVertical",
        "resolved": "\"mobile\" | \"tablet\" | boolean",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` tabs are align vertically."
      },
      "attribute": "vertical",
      "reflect": false,
      "defaultValue": "false"
    },
    "verticalColSize": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalTabsColSize",
        "resolved": "\"full\" | \"half\" | \"one-quarter\" | \"one-third\" | \"three-quarters\" | \"two-thirds\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The col size of the tabs on vertical mode."
      },
      "attribute": "vertical-col-size",
      "reflect": false,
      "defaultValue": "'one-third'"
    },
    "selectOnMobile": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the tabs are shown as a select component on mobile"
      },
      "attribute": "select-on-mobile",
      "reflect": false,
      "defaultValue": "false"
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "undefined"
    }
  }; }
  static get states() { return {
    "tabsOptions": {},
    "lineWidth": {},
    "lineOffsetLeft": {},
    "lineHeight": {},
    "lineOffsetTop": {},
    "isReady": {},
    "platform": {}
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the changes has finished."
      },
      "complexType": {
        "original": "Events.BalTabsChangeDetail",
        "resolved": "string",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../types"
          }
        }
      }
    }]; }
  static get methods() { return {
    "select": {
      "complexType": {
        "signature": "(tab: BalTabOption) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "BalTabOption": {
            "location": "import",
            "path": "./bal-tab.type"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Go to tab with the given value",
        "tags": []
      }
    },
    "renderLine": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": "Rerenders the line to mark the active tab."
          }]
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "debounce",
      "methodName": "debounceChanged"
    }, {
      "propName": "value",
      "methodName": "valueChanged"
    }]; }
  static get listeners() { return [{
      "name": "resize",
      "method": "resizeHandler",
      "target": "window",
      "capture": false,
      "passive": true
    }, {
      "name": "balPopoverPrepare",
      "method": "popoverHandler",
      "target": "window",
      "capture": false,
      "passive": false
    }]; }
}
