import { areArraysEqual } from '@baloise/web-app-utils';
import { Component, h, Host, Prop, Element, Watch, Event, Listen, Method, } from '@stencil/core';
import { stopEventBubbling } from '../../../../helpers/form-input.helpers';
import { findItemLabel, inheritAttributes, isDescendant } from '../../../../helpers/helpers';
import { BEM } from '../../../../utils/bem';
export class CheckboxGroup {
  constructor() {
    this.inputId = `bal-cg-${checkboxGroupIds++}`;
    this.inheritedAttributes = {};
    /**
     * Defines the layout of the checkbox button
     */
    this.interface = undefined;
    /**
     * If `true` it acts as the main form control
     */
    this.control = false;
    /**
     * The name of the control, which is submitted with the form data.
     */
    this.name = this.inputId;
    /**
     * Displays the checkboxes vertically
     */
    this.vertical = false;
    /**
     * If `true`, the controls will be vertically on mobile devices.
     */
    this.verticalOnMobile = false;
    /**
     * Uses the whole width
     */
    this.expanded = false;
    /**
     * If `true`, the user cannot interact with the checkboxes.
     */
    this.disabled = undefined;
    /**
     * If `true`, the user cannot interact with the checkboxes.
     */
    this.readonly = undefined;
    /**
     * The value of the control.
     */
    this.value = [];
    this.onClick = (ev) => {
      if (!this.control) {
        return;
      }
      const element = ev.target;
      if (element.href) {
        return;
      }
      ev.preventDefault();
      // toggle clicked checkbox
      const selectedCheckbox = ev.target && ev.target.closest('bal-checkbox');
      if (selectedCheckbox) {
        if (selectedCheckbox.disabled || selectedCheckbox.readonly) {
          ev.stopPropagation();
          return;
        }
      }
      // generate new value array out of the checked checkboxes
      const newValue = [];
      this.children.forEach(cb => {
        if (cb.checked) {
          newValue.push(cb.value);
        }
      });
      if (!areArraysEqual(this.value, newValue)) {
        this.value = [...newValue];
        this.balChange.emit(this.value);
      }
    };
  }
  disabledChanged(value) {
    if (this.control) {
      if (value !== undefined) {
        this.children.forEach(child => {
          child.disabled = value;
        });
      }
    }
  }
  readonlyChanged(value) {
    if (this.control) {
      if (value !== undefined) {
        this.children.forEach(child => {
          child.readonly = value;
        });
      }
    }
  }
  valueChanged(_value, oldValue) {
    if (this.control) {
      if (!areArraysEqual(this.value, oldValue)) {
        this.sync();
      }
    }
    else {
      this.sync();
    }
  }
  listenOnClick(ev) {
    if (this.control) {
      if (isDescendant(this.el, ev.target)) {
        stopEventBubbling(ev);
      }
    }
  }
  resetHandler(event) {
    const formElement = event.target;
    if (formElement === null || formElement === void 0 ? void 0 : formElement.contains(this.el)) {
      if (this.control) {
        this.value = [];
      }
      this.sync();
    }
  }
  componentWillLoad() {
    if (this.control) {
      this.inheritedAttributes = inheritAttributes(this.el, ['aria-label', 'tabindex', 'title']);
      this.disabledChanged(this.disabled);
      this.readonlyChanged(this.readonly);
    }
    this.sync();
  }
  get children() {
    return Array.from(this.el.querySelectorAll('bal-checkbox'));
  }
  /** @internal */
  async setValue(value) {
    if (this.control) {
      this.value = value;
    }
  }
  sync() {
    if (this.control) {
      const isChecked = (checkbox) => {
        for (let index = 0; index < this.value.length; index++) {
          const item = this.value[index];
          if (item.toString() === checkbox.value.toString()) {
            return true;
          }
        }
        return false;
      };
      this.children.forEach((checkbox) => {
        checkbox.checked = isChecked(checkbox);
      });
    }
    this.children.forEach((checkbox) => {
      if (this.interface) {
        checkbox.interface = this.interface;
      }
    });
  }
  render() {
    const label = findItemLabel(this.el);
    const block = BEM.block('radio-checkbox-group');
    const innerEl = block.element('inner');
    return (h(Host, Object.assign({ class: Object.assign({}, block.class()), role: "group", "aria-labelledby": label === null || label === void 0 ? void 0 : label.id, "aria-disabled": this.disabled ? 'true' : null, onClick: this.onClick }, this.inheritedAttributes),
      h("div", { class: Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, innerEl.class()), innerEl.modifier('vertical-mobile').class(this.verticalOnMobile)), innerEl.modifier('vertical').class(this.vertical)), innerEl.modifier('expanded').class(this.expanded)), innerEl.modifier('select-button').class(this.interface === 'select-button')) },
        h("slot", null))));
  }
  static get is() { return "bal-checkbox-group"; }
  static get properties() { return {
    "interface": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalCheckboxGroupInterface",
        "resolved": "\"checkbox\" | \"select-button\" | \"switch\" | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Defines the layout of the checkbox button"
      },
      "attribute": "interface",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "control": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` it acts as the main form control"
      },
      "attribute": "control",
      "reflect": false,
      "defaultValue": "false"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the control, which is submitted with the form data."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "this.inputId"
    },
    "vertical": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Displays the checkboxes vertically"
      },
      "attribute": "vertical",
      "reflect": false,
      "defaultValue": "false"
    },
    "verticalOnMobile": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the controls will be vertically on mobile devices."
      },
      "attribute": "vertical-on-mobile",
      "reflect": false,
      "defaultValue": "false"
    },
    "expanded": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Uses the whole width"
      },
      "attribute": "expanded",
      "reflect": false,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "If `true`, the user cannot interact with the checkboxes."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "If `true`, the user cannot interact with the checkboxes."
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "value": {
      "type": "unknown",
      "mutable": true,
      "complexType": {
        "original": "any[]",
        "resolved": "any[]",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value of the control."
      },
      "defaultValue": "[]"
    }
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the checked property has changed."
      },
      "complexType": {
        "original": "Events.BalCheckboxGroupChangeDetail",
        "resolved": "any[]",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../../types"
          }
        }
      }
    }]; }
  static get methods() { return {
    "setValue": {
      "complexType": {
        "signature": "(value: any[]) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": undefined
          }]
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "disabled",
      "methodName": "disabledChanged"
    }, {
      "propName": "readonly",
      "methodName": "readonlyChanged"
    }, {
      "propName": "value",
      "methodName": "valueChanged"
    }]; }
  static get listeners() { return [{
      "name": "balChange",
      "method": "listenOnClick",
      "target": "document",
      "capture": true,
      "passive": false
    }, {
      "name": "reset",
      "method": "resetHandler",
      "target": "document",
      "capture": true,
      "passive": false
    }]; }
}
let checkboxGroupIds = 0;
