import { Component, Host, h, Element, State, Prop, Event, Method, Watch, Listen, } from '@stencil/core';
import { addDays, subDays, isBefore, isAfter, getDate, getYear, getMonth, addYears, subYears, startOfWeek, isSameDay, isWithinInterval, isSameWeek, isSameMonth, } from 'date-fns';
import { debounceEvent, findItemLabel, inheritAttributes } from '../../../helpers/helpers';
import { isSpaceKey, parse, format, isValidIsoString, now, formatDateString, isEnterKey, dateSeparator, } from '@baloise/web-app-utils';
import isNil from 'lodash.isnil';
import { ACTION_KEYS, isCtrlOrCommandKey, NUMBER_KEYS } from '../../../constants/keys.constant';
import { i18nDate } from './bal-datepicker.i18n';
import { detachComponentToConfig, defaultConfig, attachComponentToConfig, useBalConfig, defaultLocale, } from '../../../config';
import { getInputTarget, inputHandleFocus, inputHandleHostClick, inputListenOnClick, inputSetBlur, inputSetFocus, stopEventBubbling, } from '../../../helpers/form-input.helpers';
import { preventDefault } from '../bal-select/utils/utils';
import { BEM } from '../../../utils/bem';
import { isPlatform } from '../../../utils/platform';
export class Datepicker {
  constructor() {
    this.inputId = `bal-dp-${datepickerIds++}`;
    this.inheritedAttributes = {};
    this.inputValue = this.value;
    this.language = defaultConfig.language;
    this.region = defaultConfig.region;
    this.isMobile = isPlatform('mobile');
    this.hasFocus = false;
    this.isPopoverOpen = false;
    this.selectedDate = '';
    this.pointerDate = {
      year: getYear(now()),
      month: getMonth(now()),
      day: getDate(now()),
    };
    /**
     * The name of the control, which is submitted with the form data.
     */
    this.name = this.inputId;
    /**
     * If `true` the component gets a invalid style.
     */
    this.invalid = false;
    /**
     * @deprecated Define the locale of month and day names.
     */
    this.locale = '';
    /**
     * Set this to `true` when the component is placed on a dark background.
     */
    this.inverted = false;
    /**
     * If `true` the attribute required is added to the native input.
     */
    this.required = false;
    /**
     * If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants.
     */
    this.disabled = false;
    /**
     * If `true` the element can not mutated, meaning the user can not edit the control.
     */
    this.readonly = false;
    /**
     * Closes the datepicker popover after selection
     */
    this.closeOnSelect = true;
    /**
     * If `true` the datepicker only open on click of the icon
     */
    this.triggerIcon = false;
    /**
     * Set the amount of time, in milliseconds, to wait to trigger the `ionChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`.
     */
    this.debounce = 0;
    /**
     * Callback to determine which date in the datepicker should be selectable.
     */
    this.allowedDates = undefined;
    this.onIconClick = (event) => {
      if (!this.disabled && !this.readonly) {
        this.popoverElement.toggle();
      }
      stopEventBubbling(event);
      this.balClick.emit(event);
    };
    this.onInputClick = (event) => {
      if (!this.triggerIcon && !this.disabled && !this.readonly) {
        this.popoverElement.toggle();
      }
      stopEventBubbling(event);
      if (!this.triggerIcon) {
        this.balClick.emit(event);
      }
    };
    this.onPopoverChange = (event) => {
      this.isPopoverOpen = event.detail;
      preventDefault(event);
      if (!this.isPopoverOpen) {
        this.balBlur.emit();
      }
    };
    this.onInput = (event) => {
      const input = getInputTarget(event);
      if (input) {
        this.inputValue = input.value;
        if (this.inputValue && this.inputValue.length >= 6) {
          const date = parse(this.inputValue, this.getLocale());
          const dateString = formatDateString(date);
          if (isValidIsoString(dateString)) {
            this.selectedDate = dateString;
            this.updatePointerDates();
          }
        }
      }
      this.balInput.emit(this.inputValue);
    };
    this.onInputChange = (event) => {
      const inputValue = event.target.value;
      const date = parse(inputValue, this.getLocale());
      const dateString = formatDateString(date);
      const formattedValue = format(this.getLocale(), date);
      this.nativeInput.value = formattedValue;
      this.updateValue(dateString);
      this.updatePointerDates();
    };
    this.onClickDateCell = (cell) => {
      if (!cell.isDisabled) {
        this.select(cell.dateString);
      }
    };
    this.onInputKeyUp = (event) => {
      if (isSpaceKey(event) && !this.triggerIcon) {
        if (this.isPopoverOpen) {
          this.close();
        }
        else {
          this.open();
        }
      }
      if (isEnterKey(event) && !this.triggerIcon) {
        const date = parse(this.nativeInput.value, this.getLocale());
        const dateString = formatDateString(date);
        if (this.isPopoverOpen) {
          if (this.value === dateString) {
            this.close();
          }
        }
      }
    };
    this.onInputKeyDown = (event) => {
      const separator = dateSeparator(this.getLocale());
      const allowedKeys = [...NUMBER_KEYS, separator, ...ACTION_KEYS];
      if (!isCtrlOrCommandKey(event) && allowedKeys.indexOf(event.key) < 0) {
        event.preventDefault();
        event.stopPropagation();
      }
      if (event.key === 'Tab') {
        this.close();
      }
    };
    this.onMonthSelect = (event) => {
      const inputValue = event.target.value;
      this.pointerDate = Object.assign(Object.assign({}, this.pointerDate), { month: parseInt(inputValue, 10) });
    };
    this.onYearSelect = (event) => {
      const inputValue = event.target.value;
      const yearValue = parseInt(inputValue, 10);
      this.pointerDate = Object.assign(Object.assign({}, this.pointerDate), { year: yearValue });
    };
    this.onInputFocus = (event) => inputHandleFocus(this, event);
    this.onInputBlur = (event) => {
      preventDefault(event);
      this.hasFocus = false;
    };
    this.handleClick = (event) => inputHandleHostClick(this, event);
  }
  watchLocaleHandler() {
    console.warn('[DEPRECATED] - Please use the window.BaloiseDesignSystem.config instead');
    this.migrateLocale();
  }
  migrateLocale() {
    if (this.locale !== '') {
      this.language = this.locale;
    }
  }
  debounceChanged() {
    this.balChange = debounceEvent(this.balChange, this.debounce);
  }
  /**
   * Update the native input element when the value changes
   */
  valueChanged() {
    this.selectedDate = this.value;
    this.updatePointerDates();
  }
  listenOnClick(event) {
    inputListenOnClick(this, event);
  }
  resetHandler(event) {
    const formElement = event.target;
    if (formElement === null || formElement === void 0 ? void 0 : formElement.contains(this.el)) {
      if (this.resetHandlerTimer) {
        clearTimeout(this.resetHandlerTimer);
      }
      this.resetHandlerTimer = setTimeout(() => {
        if (this.initialValue) {
          this.nativeInput.value = format(this.getLocale(), parse(this.initialValue));
        }
        else {
          this.nativeInput.value = '';
        }
        this.selectedDate = this.initialValue;
        this.updateValue(this.initialValue, false);
        this.updatePointerDates();
      }, 0);
    }
  }
  async resizeHandler() {
    this.isMobile = isPlatform('mobile');
  }
  connectedCallback() {
    this.debounceChanged();
    attachComponentToConfig(this);
    this.initialValue = this.value;
    this.migrateLocale();
  }
  componentDidLoad() {
    this.inputValue = this.value;
  }
  componentWillLoad() {
    this.inheritedAttributes = inheritAttributes(this.el, ['aria-label', 'tabindex', 'title']);
    this.selectedDate = this.value;
    this.updatePointerDates();
    this.updateValue(this.value, false);
  }
  disconnectedCallback() {
    detachComponentToConfig(this);
  }
  configChanged(config) {
    this.language = config.language;
    this.region = config.region;
  }
  /**
   * Opens the popover
   */
  async open() {
    if (!this.disabled && this.popoverElement) {
      this.popoverElement.present();
    }
  }
  /**
   * Closes the popover
   */
  async close() {
    if (!this.disabled && this.popoverElement) {
      this.popoverElement.dismiss();
    }
  }
  /**
   * Selects an option
   */
  async select(dateString) {
    var _a;
    this.nativeInput.value = format(this.getLocale(), parse(dateString));
    this.updateValue(dateString);
    this.updatePointerDates();
    if (this.closeOnSelect) {
      await ((_a = this.popoverElement) === null || _a === void 0 ? void 0 : _a.toggle());
    }
  }
  /**
   * Sets focus on the native `input`. Use this method instead of the global
   * `input.focus()`.
   */
  async setFocus() {
    inputSetFocus(this);
  }
  /**
   * Sets blur on the native `input`. Use this method instead of the global
   * `input.blur()`.
   * @internal
   */
  async setBlur() {
    inputSetBlur(this);
  }
  /**
   * Returns the native `<input>` element used under the hood.
   */
  getInputElement() {
    return Promise.resolve(this.nativeInput);
  }
  updatePointerDates() {
    let date = now();
    date.setDate(1);
    if (this.selectedDate) {
      date = parse(this.selectedDate);
    }
    else {
      if (this.defaultDate) {
        date = parse(this.defaultDate);
      }
    }
    if (this.min) {
      const minDate = parse(this.min);
      if (minDate && date < minDate) {
        date = minDate;
      }
    }
    if (this.max) {
      const maxDate = parse(this.max);
      if (maxDate && date > maxDate) {
        date = maxDate;
      }
    }
    this.pointerDate = {
      year: getYear(date),
      month: getMonth(date),
      day: getDate(date),
    };
  }
  updateValue(dateString, isHuman = true) {
    if (!isValidIsoString(dateString)) {
      this.selectedDate = undefined;
      this.value = undefined;
      if (this.nativeInput) {
        this.nativeInput.value = '';
      }
    }
    if (this.value !== dateString) {
      this.value = dateString;
      if (isHuman) {
        this.balChange.emit(this.value);
      }
    }
  }
  get minYear() {
    if (this.min) {
      return parseInt(this.min.substring(0, 4), 10);
    }
    return this.minYearProp ? this.minYearProp : getYear(subYears(now(), 100));
  }
  get maxYear() {
    if (this.max) {
      return parseInt(this.max.substring(0, 4), 10);
    }
    return this.maxYearProp ? this.maxYearProp : getYear(addYears(now(), 100));
  }
  get years() {
    let years = Array.from({ length: this.maxYear - this.minYear + 1 }, (_, index) => this.minYear + index);
    if (this.min && this.pointerDate.month === getMonth(parse(this.min))) {
      const minYear = getYear(parse(this.min));
      years = years.filter(y => y >= minYear);
    }
    if (this.max && this.pointerDate.month === getMonth(parse(this.max))) {
      const maxYear = getYear(parse(this.max));
      years = years.filter(y => y <= maxYear);
    }
    return years;
  }
  get months() {
    const monthNames = i18nDate[this.language].monthsShort;
    let months = monthNames.map((name, index) => ({ name, index }));
    if (this.min && this.pointerDate.year === getYear(parse(this.min))) {
      const minMonth = parseInt(this.min.substring(5, 7), 10) - 1;
      months = months.filter(month => month.index >= minMonth);
    }
    if (this.max && this.pointerDate.year === getYear(parse(this.max))) {
      const maxMonth = parseInt(this.max.substring(5, 7), 10) - 1;
      months = months.filter(month => month.index <= maxMonth);
    }
    return months;
  }
  get weekDays() {
    const translations = [...i18nDate[this.language].weekdaysMin];
    translations.push(translations.shift() || '');
    return translations;
  }
  get firstDateOfBox() {
    const date = new Date(this.pointerDate.year, this.pointerDate.month, 1);
    return startOfWeek(date, { weekStartsOn: 1 });
  }
  getLocale() {
    const config = useBalConfig();
    return (config && config.locale) || defaultLocale;
  }
  get calendarGrid() {
    const weekDatePointer = this.firstDateOfBox;
    const dayDatePointer = this.firstDateOfBox;
    let calendar = []; // eslint-disable-line
    do {
      let row = []; // eslint-disable-line
      do {
        row = [
          ...row,
          {
            date: new Date(dayDatePointer),
            display: format(this.getLocale(), dayDatePointer),
            dateString: formatDateString(dayDatePointer),
            label: getDate(dayDatePointer).toString(),
            isToday: isSameDay(dayDatePointer, now()),
            isSelected: parse(this.selectedDate) !== undefined &&
              isSameDay(dayDatePointer, parse(this.selectedDate)),
            isDisabled: !this.getAllowedDates(dayDatePointer) || !this.isDateInRange(dayDatePointer),
            isOutdated: this.pointerDate.month !== dayDatePointer.getMonth() || !this.isDateInRange(dayDatePointer),
          },
        ];
        dayDatePointer.setDate(dayDatePointer.getDate() + 1);
      } while (isSameWeek(dayDatePointer, weekDatePointer, { weekStartsOn: 1 }));
      calendar = [...calendar, row];
      weekDatePointer.setDate(weekDatePointer.getDate() + 7);
    } while (isSameMonth(new Date(this.pointerDate.year, this.pointerDate.month, this.pointerDate.day), dayDatePointer));
    return calendar;
  }
  getAllowedDates(dayDatePointer) {
    if (isNil(this.allowedDates)) {
      return true;
    }
    return this.allowedDates(formatDateString(dayDatePointer));
  }
  render() {
    const block = BEM.block('datepicker');
    const native = block.element('native');
    const popup = block.element('popup');
    const popupBody = popup.element('body');
    const popupFooter = popup.element('footer');
    return (h(Host, { onClick: this.handleClick, "aria-disabled": this.disabled ? 'true' : null, class: Object.assign(Object.assign({}, block.class()), block.modifier('disabled').class(this.disabled || this.readonly)) },
      h("input", { type: "date", class: Object.assign({}, native.class()), name: this.name, value: this.value, min: this.min, max: this.max, tabindex: -1 }),
      h("bal-popover", { onBalChange: this.onPopoverChange, ref: el => (this.popoverElement = el) },
        this.renderInput(),
        h("bal-popover-content", { spaceless: true, contentMinWidth: 392 },
          h("div", { class: Object.assign({}, popup.class()) },
            this.renderHeader(),
            h("div", { class: Object.assign({}, popupBody.class()) }, this.renderGrid()),
            h("div", { class: Object.assign({}, popupFooter.class()) },
              h("slot", null)))))));
  }
  renderInput() {
    const labelId = this.inputId + '-lbl';
    const label = findItemLabel(this.el);
    if (label) {
      label.id = labelId;
      label.htmlFor = this.inputId;
    }
    return (h("div", { "bal-popover-trigger": true, class: "control" },
      h("bal-input-group", { disabled: this.disabled || this.readonly, invalid: this.invalid },
        h("input", Object.assign({ class: {
            'input': true,
            'data-test-input': true,
            'is-clickable': !this.disabled && !this.triggerIcon && !this.readonly,
            'is-inverted': this.inverted,
            'is-disabled': this.disabled || this.readonly,
            'is-danger': this.invalid,
          }, ref: el => (this.nativeInput = el), id: this.inputId, "aria-labelledby": labelId, type: "text", maxlength: "10", autoComplete: "off", value: format(this.getLocale(), parse(this.value || '')), required: this.required, disabled: this.disabled, readonly: this.readonly, placeholder: this.placeholder, onKeyDown: e => this.onInputKeyDown(e), onKeyUp: e => this.onInputKeyUp(e), onInput: this.onInput, onClick: this.onInputClick, onChange: this.onInputChange, onBlur: this.onInputBlur, onFocus: this.onInputFocus }, this.inheritedAttributes)),
        h("bal-icon", { class: {
            'datepicker-trigger-icon': true,
            'is-clickable': !this.disabled && !this.readonly,
          }, "is-right": true, color: this.disabled || this.readonly ? 'grey' : this.invalid ? 'danger' : 'primary', inverted: this.inverted, name: "date", onClick: this.onIconClick }))));
  }
  renderGrid() {
    const block = BEM.block('datepicker-grid');
    const rowEl = block.element('row');
    const cellEl = block.element('cell');
    return (h("div", null,
      this.renderWeekDayHeader(),
      h("div", { class: Object.assign({}, block.class()) }, this.calendarGrid.map(row => (h("div", { class: Object.assign({}, rowEl.class()) }, row.map(cell => (h("button", { type: "button", "data-date": cell.dateString, onClick: () => this.onClickDateCell(cell), disabled: cell.isDisabled, class: Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, cellEl.class()), { 'button': true, 'is-text': !cell.isDisabled && !cell.isSelected, 'is-primary': cell.isSelected && cell.isSelected, 'is-disabled': cell.isDisabled || cell.isOutdated }), cellEl.modifier('today').class(cell.isToday)), cellEl.modifier('selectable').class(!cell.isDisabled && !cell.isOutdated)), cellEl.modifier('disabled').class(cell.isDisabled || cell.isOutdated)), cellEl.modifier('selected').class(cell.isSelected)) },
        h("span", null, cell.label))))))))));
  }
  renderWeekDayHeader() {
    const block = BEM.block('datepicker-grid');
    const headerEl = block.element('header');
    const cellEl = block.element('cell');
    return (h("header", { class: Object.assign({}, headerEl.class()) }, this.weekDays.map(weekday => (h("div", { class: Object.assign(Object.assign({}, cellEl.class()), cellEl.modifier('header').class()) },
      h("span", null, weekday))))));
  }
  renderHeader() {
    const block = BEM.block('datepicker-pagination');
    const innerEl = block.element('inner');
    const monthAndYearEl = block.element('month-and-year');
    const selectEl = monthAndYearEl.element('select');
    return (h("header", { class: Object.assign({}, block.class()) },
      h("div", { class: Object.assign({}, innerEl.class()) },
        h("bal-button", { square: true, size: this.isMobile ? 'small' : '', color: "info", icon: "nav-go-left", disabled: this.isPreviousMonthDisabled, onClick: () => this.previousMonth() }),
        h("div", { class: Object.assign({}, monthAndYearEl.class()) },
          h("div", { class: Object.assign(Object.assign({}, selectEl.class()), selectEl.modifier('month').class()) },
            h("div", { class: "select" },
              h("select", { class: "", onInput: this.onMonthSelect }, this.months.map(month => (h("option", { value: month.index, selected: this.pointerDate.month === month.index }, month.name)))))),
          h("div", { class: Object.assign(Object.assign({}, selectEl.class()), selectEl.modifier('year').class()) },
            h("div", { class: "select" },
              h("select", { onInput: this.onYearSelect }, this.years.map(year => (h("option", { value: year, selected: this.pointerDate.year === year }, year))))))),
        h("bal-button", { square: true, size: this.isMobile ? 'small' : '', color: "info", icon: "nav-go-right", disabled: this.isNextMonthDisabled, onClick: () => this.nextMonth() }))));
  }
  previousMonth() {
    if (!this.isPreviousMonthDisabled) {
      if (this.pointerDate.year === this.minYear && this.pointerDate.month === 0) {
        return;
      }
      this.pointerDate = this.calcPreviousMonth();
    }
  }
  nextMonth() {
    if (!this.isNextMonthDisabled) {
      if (this.pointerDate.year === this.maxYear && this.pointerDate.month === 11) {
        return;
      }
      this.pointerDate = this.calcNextMonth();
    }
  }
  calcPreviousMonth() {
    if (this.pointerDate.month === 0) {
      return Object.assign(Object.assign({}, this.pointerDate), { year: this.pointerDate.year - 1, month: 11 });
    }
    else {
      return Object.assign(Object.assign({}, this.pointerDate), { month: this.pointerDate.month - 1 });
    }
  }
  calcNextMonth() {
    if (this.pointerDate.month === 11) {
      return Object.assign(Object.assign({}, this.pointerDate), { year: this.pointerDate.year + 1, month: 0 });
    }
    else {
      return Object.assign(Object.assign({}, this.pointerDate), { month: this.pointerDate.month + 1 });
    }
  }
  lastDayOfMonth(year, month) {
    const d = new Date(year, month + 1, 0);
    return getDate(d);
  }
  get isPreviousMonthDisabled() {
    if (this.min) {
      const minDate = parse(this.min);
      const lastDayOfMonth = this.lastDayOfMonth(this.calcPreviousMonth().year, this.calcPreviousMonth().month);
      const beforeDate = new Date(this.calcPreviousMonth().year, this.calcPreviousMonth().month, lastDayOfMonth);
      return isBefore(beforeDate, subDays(minDate, 1));
    }
    return false;
  }
  get isNextMonthDisabled() {
    if (this.max) {
      const maxDate = parse(this.max);
      const beforeDate = new Date(this.calcNextMonth().year, this.calcNextMonth().month, 1);
      return isAfter(beforeDate, addDays(maxDate, 1));
    }
    return false;
  }
  isDateInRange(cellDate) {
    const parsedCellDate = parse(formatDateString(cellDate));
    if (this.min && this.max && this.max > this.min) {
      return isWithinInterval(parsedCellDate, {
        start: parse(this.min),
        end: parse(this.max),
      });
    }
    if (this.min) {
      return isAfter(parsedCellDate, parse(this.min));
    }
    if (this.max) {
      return isBefore(parsedCellDate, addDays(parse(this.max), 1));
    }
    return true;
  }
  static get is() { return "bal-datepicker"; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the control, which is submitted with the form data."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "this.inputId"
    },
    "invalid": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the component gets a invalid style."
      },
      "attribute": "invalid",
      "reflect": false,
      "defaultValue": "false"
    },
    "locale": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'en' | 'de' | 'fr' | 'it' | ''",
        "resolved": "\"\" | \"de\" | \"en\" | \"fr\" | \"it\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Define the locale of month and day names."
          }],
        "text": ""
      },
      "attribute": "locale",
      "reflect": false,
      "defaultValue": "''"
    },
    "inverted": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set this to `true` when the component is placed on a dark background."
      },
      "attribute": "inverted",
      "reflect": false,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the attribute required is added to the native input."
      },
      "attribute": "required",
      "reflect": false,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the element can not mutated, meaning the user can not edit the control."
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "false"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The text to display when the select is empty."
      },
      "attribute": "placeholder",
      "reflect": false
    },
    "min": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The minimum datetime allowed. Value must be a date string\nfollowing the\n[ISO 8601 datetime format standard](https://www.w3.org/TR/NOTE-datetime),\nsuch as `1996-12-19`. The format does not have to be specific to an exact\ndatetime. For example, the minimum could just be the year, such as `1994`.\nDefaults to the beginning of the year, 100 years ago from today."
      },
      "attribute": "min",
      "reflect": false
    },
    "max": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The maximum datetime allowed. Value must be a date string\nfollowing the\n[ISO 8601 datetime format standard](https://www.w3.org/TR/NOTE-datetime),\n`1996-12-19`. The format does not have to be specific to an exact\ndatetime. For example, the maximum could just be the year, such as `1994`.\nDefaults to the end of this year."
      },
      "attribute": "max",
      "reflect": false
    },
    "closeOnSelect": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Closes the datepicker popover after selection"
      },
      "attribute": "close-on-select",
      "reflect": false,
      "defaultValue": "true"
    },
    "triggerIcon": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the datepicker only open on click of the icon"
      },
      "attribute": "trigger-icon",
      "reflect": false,
      "defaultValue": "false"
    },
    "minYearProp": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Earliest year available for selection"
      },
      "attribute": "min-year",
      "reflect": false
    },
    "maxYearProp": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Latest year available for selection"
      },
      "attribute": "max-year",
      "reflect": false
    },
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set the amount of time, in milliseconds, to wait to trigger the `ionChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`."
      },
      "attribute": "debounce",
      "reflect": false,
      "defaultValue": "0"
    },
    "defaultDate": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The date to defines where the datepicker popup starts. The prop accepts ISO 8601 date strings (YYYY-MM-DD)."
      },
      "attribute": "default-date",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The value of the form field, which accepts ISO 8601 date strings (YYYY-MM-DD)."
      },
      "attribute": "value",
      "reflect": false
    },
    "allowedDates": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Props.BalDatepickerCallback | undefined",
        "resolved": "((dateString: string) => boolean) | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Callback to determine which date in the datepicker should be selectable."
      },
      "defaultValue": "undefined"
    }
  }; }
  static get states() { return {
    "language": {},
    "region": {},
    "isMobile": {},
    "hasFocus": {},
    "isPopoverOpen": {},
    "selectedDate": {},
    "pointerDate": {}
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a option got selected."
      },
      "complexType": {
        "original": "Events.BalDatepickerChangeDetail",
        "resolved": "string | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }, {
      "method": "balInput",
      "name": "balInput",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a keyboard input occurred."
      },
      "complexType": {
        "original": "Events.BalDatepickerInputDetail",
        "resolved": "string | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }, {
      "method": "balBlur",
      "name": "balBlur",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input loses focus."
      },
      "complexType": {
        "original": "FocusEvent",
        "resolved": "FocusEvent",
        "references": {
          "FocusEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balFocus",
      "name": "balFocus",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input has focus."
      },
      "complexType": {
        "original": "FocusEvent",
        "resolved": "FocusEvent",
        "references": {
          "FocusEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balClick",
      "name": "balClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input has clicked."
      },
      "complexType": {
        "original": "MouseEvent",
        "resolved": "MouseEvent",
        "references": {
          "MouseEvent": {
            "location": "global"
          }
        }
      }
    }]; }
  static get methods() { return {
    "open": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Opens the popover",
        "tags": []
      }
    },
    "close": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Closes the popover",
        "tags": []
      }
    },
    "select": {
      "complexType": {
        "signature": "(dateString: string) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Selects an option",
        "tags": []
      }
    },
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets focus on the native `input`. Use this method instead of the global\n`input.focus()`.",
        "tags": []
      }
    },
    "setBlur": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets blur on the native `input`. Use this method instead of the global\n`input.blur()`.",
        "tags": [{
            "name": "internal",
            "text": undefined
          }]
      }
    },
    "getInputElement": {
      "complexType": {
        "signature": "() => Promise<HTMLInputElement>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLInputElement>"
      },
      "docs": {
        "text": "Returns the native `<input>` element used under the hood.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "locale",
      "methodName": "watchLocaleHandler"
    }, {
      "propName": "debounce",
      "methodName": "debounceChanged"
    }, {
      "propName": "value",
      "methodName": "valueChanged"
    }]; }
  static get listeners() { return [{
      "name": "click",
      "method": "listenOnClick",
      "target": "document",
      "capture": true,
      "passive": false
    }, {
      "name": "reset",
      "method": "resetHandler",
      "target": "document",
      "capture": true,
      "passive": false
    }, {
      "name": "resize",
      "method": "resizeHandler",
      "target": "window",
      "capture": false,
      "passive": true
    }]; }
}
let datepickerIds = 0;
