import { Component, Host, h, Prop, Element, Listen, State, Event, Method, Watch } from '@stencil/core';
import fileSize from 'filesize.js';
import { areArraysEqual } from '@baloise/web-app-utils';
import { FileUploadRejectionReason } from './bal-file-upload.type';
import { inputHandleBlur, inputHandleFocus, inputHandleHostClick, inputSetBlur, inputSetFocus, stopEventBubbling, } from '../../../helpers/form-input.helpers';
export class FileUpload {
  constructor() {
    this.uploadId = `bal-upload-${UploadIds++}`;
    this.bundleSize = 0;
    this.isOver = false;
    this.files = [];
    this.hasFocus = false;
    /**
     * The name of the control, which is submitted with the form data.
     */
    this.name = this.uploadId;
    /**
     * Label of the drop area.
     */
    this.label = 'Choose or drop a file...';
    /**
     * If `true` multiple file upload is possible.
     */
    this.multiple = true;
    /**
     * If `true` below the drop-down area it generates a file list.
     */
    this.hasFileList = true;
    /**
     * If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants.
     */
    this.disabled = false;
    /**
     * If `true`, the user must fill in a value before submitting a form.
     */
    this.required = false;
    /**
     * If `true` the component gets a invalid style.
     */
    this.invalid = false;
    /**
     * If `true` the element can not mutated, meaning the user can not edit the control.
     */
    this.readonly = false;
    /**
     * Accepted MIME-Types like `image/png,image/jpeg`.
     */
    this.accept = '';
    /**
     * Input value.
     */
    this.value = [];
    /**
     * If `true` the file upload is disabled and shows a spinner
     */
    this.loading = false;
    this.handleFiles = (files) => {
      if (!this.disabled) {
        const list = [...this.files];
        const filesAdded = [];
        for (let index = 0; index < files.length; index++) {
          const file = files.item(index);
          if (file) {
            const rejectReasons = [];
            if (this.accept && this.accept.split(' ').join('').split(',').indexOf(file.type) === -1) {
              rejectReasons.push(FileUploadRejectionReason.BAD_EXTENSION);
            }
            if (this.maxFileSize && file.size > this.maxFileSize) {
              rejectReasons.push(FileUploadRejectionReason.FILE_TOO_BIG);
            }
            const transactionFileSizeSum = this.files.map(f => f.size).reduce((a, b) => a + b, 0);
            const bundleSize = file.size + transactionFileSizeSum;
            if (this.maxBundleSize && bundleSize > this.maxBundleSize) {
              rejectReasons.push(FileUploadRejectionReason.FILE_SIZE_SUM_TOO_BIG);
            }
            if (this.maxFiles && list.length + 1 > this.maxFiles) {
              rejectReasons.push(FileUploadRejectionReason.TOO_MANY_FILES);
            }
            const duplicatedFiles = list.filter(f => f.size === file.size && f.name === file.name && f.type === file.type);
            if (duplicatedFiles.length > 0) {
              rejectReasons.push(FileUploadRejectionReason.DUPLICATED_FILE);
            }
            if (rejectReasons.length > 0) {
              this.balRejectedFileEventEmitter.emit({
                file: file,
                reasons: rejectReasons,
              });
            }
            else {
              list.push(file);
              filesAdded.push(file);
            }
          }
        }
        if (this.files.length !== list.length) {
          this.files = [...list];
          this.balChange.emit(this.files);
        }
        if (filesAdded.length > 0) {
          this.balFilesAddedEmitter.emit(filesAdded);
        }
        this.setFileList();
      }
    };
    this.onInputChange = () => {
      var _a;
      if ((_a = this.nativeInput) === null || _a === void 0 ? void 0 : _a.files) {
        const files = this.nativeInput.files;
        this.handleFiles(files);
      }
    };
    this.setFileList = () => {
      const fileList = new DataTransfer();
      this.files.forEach(el => fileList.items.add(el));
      this.nativeInput.files = fileList.files;
    };
    this.onInputFocus = (event) => inputHandleFocus(this, event);
    this.onInputBlur = (event) => inputHandleBlur(this, event);
    this.handleClick = (event) => inputHandleHostClick(this, event);
    this.onInputClick = (event) => {
      this.balClick.emit(event);
    };
  }
  onValueChange() {
    if (!areArraysEqual(this.files, this.value)) {
      this.files = this.value;
    }
  }
  dragenterHandler() {
    if (!this.disabled && !this.readonly) {
      this.isOver = true;
    }
  }
  dragoverHandler() {
    if (!this.disabled && !this.readonly) {
      this.isOver = true;
    }
  }
  dragleaveHandler() {
    if (!this.disabled && !this.readonly) {
      this.isOver = false;
    }
  }
  dropHandler(event) {
    if (!this.disabled && !this.readonly) {
      this.isOver = false;
      const dataTransfer = event.dataTransfer;
      if (dataTransfer) {
        this.handleFiles(dataTransfer.files);
      }
    }
  }
  componentWillLoad() {
    this.onValueChange();
  }
  componentDidLoad() {
    ;
    ['dragenter', 'dragover', 'dragleave', 'drop'].forEach(eventName => {
      this.el.addEventListener(eventName, stopEventBubbling, {
        passive: false,
      });
      document.body.addEventListener(eventName, stopEventBubbling, {
        passive: false,
      });
    });
  }
  disconnectedCallback() {
    ;
    ['dragenter', 'dragover', 'dragleave', 'drop'].forEach(eventName => {
      this.el.removeEventListener(eventName, stopEventBubbling, false);
      document.body.removeEventListener(eventName, stopEventBubbling, false);
    });
  }
  /**
   * Sets the file list to an empty list
   */
  async clear() {
    this.files = [];
  }
  /**
   * Sets focus on the native `input`. Use this method instead of the global
   * `input.focus()`.
   */
  async setFocus() {
    inputSetFocus(this);
  }
  /**
   * Sets blur on the native `input`. Use this method instead of the global
   * `input.blur()`.
   * @internal
   */
  async setBlur() {
    inputSetBlur(this);
  }
  /**
   * Returns the native `<input>` element used under the hood.
   */
  getInputElement() {
    return Promise.resolve(this.nativeInput);
  }
  removeFile(indexToRemove) {
    if (!this.disabled && !this.readonly) {
      const list = [];
      const removedFiles = [];
      for (let index = 0; index < this.files.length; index++) {
        if (index !== indexToRemove) {
          list.push(this.files[index]);
        }
        else {
          removedFiles.push(this.files[index]);
        }
      }
      this.files = [...list];
      this.balChange.emit(this.files);
      this.balFilesRemovedEmitter.emit(removedFiles);
      this.setFileList();
    }
  }
  render() {
    const FileList = () => (h("bal-card", { flat: true, class: "mt-4", style: { display: this.files.length ? 'block' : 'none' } },
      h("bal-list", { disabled: this.disabled || this.loading || this.readonly, border: true, size: "large", class: "p-0" }, this.files.map((file, index) => (h("bal-list-item", { disabled: this.disabled || this.loading || this.readonly },
        h("bal-list-item-icon", null,
          h("bal-icon", { name: "document" })),
        h("bal-list-item-content", null,
          h("bal-list-item-title", null, file.name),
          h("bal-list-item-subtitle", null, this.subTitle ? this.subTitle(file) : fileSize(file.size))),
        h("bal-list-item-icon", { right: true, class: {
            'file-remove': true,
            'is-clickable': !this.disabled && !this.readonly,
          }, onClick: () => this.removeFile(index) },
          h("bal-icon", { name: "trash", color: this.disabled || this.loading || this.readonly ? 'grey' : 'danger' }))))))));
    return (h(Host, { onClick: this.handleClick, class: {
        'bal-file-upload': true,
      } },
      h("div", { class: {
          'file': true,
          'is-disabled': this.disabled || this.readonly || this.loading,
          'is-danger': this.invalid,
        } },
        h("label", { class: ['file-label', this.disabled || this.loading || this.readonly ? 'is-disabled' : ''].join(' ') },
          h("input", { class: "file-input", type: "file", name: this.name, multiple: this.multiple, disabled: this.disabled || this.loading, readonly: this.readonly, required: this.required, accept: this.accept, onClick: this.onInputClick, onChange: this.onInputChange, onFocus: this.onInputFocus, onBlur: this.onInputBlur, ref: el => (this.nativeInput = el) }),
          this.loading ? (h("span", { class: "file-cta" },
            h("bal-spinner", null))) : (h("span", { class: "file-cta" },
            h("span", { class: "file-icon" },
              h("bal-icon", { name: "upload", size: "medium", color: this.disabled || this.loading || this.readonly ? 'grey-light' : this.invalid ? 'danger' : 'blue' })),
            h("span", { class: "file-label" }, this.label))))),
      this.hasFileList ? h(FileList, null) : ''));
  }
  static get is() { return "bal-file-upload"; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the control, which is submitted with the form data."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "this.uploadId"
    },
    "label": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Label of the drop area."
      },
      "attribute": "label",
      "reflect": false,
      "defaultValue": "'Choose or drop a file...'"
    },
    "multiple": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` multiple file upload is possible."
      },
      "attribute": "multiple",
      "reflect": false,
      "defaultValue": "true"
    },
    "hasFileList": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` below the drop-down area it generates a file list."
      },
      "attribute": "has-file-list",
      "reflect": false,
      "defaultValue": "true"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the user must fill in a value before submitting a form."
      },
      "attribute": "required",
      "reflect": false,
      "defaultValue": "false"
    },
    "invalid": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the component gets a invalid style."
      },
      "attribute": "invalid",
      "reflect": false,
      "defaultValue": "false"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the element can not mutated, meaning the user can not edit the control."
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "false"
    },
    "accept": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Accepted MIME-Types like `image/png,image/jpeg`."
      },
      "attribute": "accept",
      "reflect": false,
      "defaultValue": "''"
    },
    "maxFiles": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Allowed number of files in the bundle."
      },
      "attribute": "max-files",
      "reflect": false
    },
    "maxFileSize": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Allowed max file size in bytes."
      },
      "attribute": "max-file-size",
      "reflect": false
    },
    "maxBundleSize": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Allowed max bundle size in bytes."
      },
      "attribute": "max-bundle-size",
      "reflect": false
    },
    "value": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "File[]",
        "resolved": "File[]",
        "references": {
          "File": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Input value."
      },
      "defaultValue": "[]"
    },
    "loading": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the file upload is disabled and shows a spinner"
      },
      "attribute": "loading",
      "reflect": false,
      "defaultValue": "false"
    },
    "subTitle": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "(file: File) => string",
        "resolved": "((file: File) => string) | undefined",
        "references": {
          "File": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Overrides the default subtitle file size"
      }
    }
  }; }
  static get states() { return {
    "isOver": {},
    "files": {},
    "hasFocus": {}
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Triggers when a file is added or removed."
      },
      "complexType": {
        "original": "File[]",
        "resolved": "File[]",
        "references": {
          "File": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balFilesAddedEmitter",
      "name": "balFilesAdded",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Triggers when a file is added."
      },
      "complexType": {
        "original": "File[]",
        "resolved": "File[]",
        "references": {
          "File": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balFilesRemovedEmitter",
      "name": "balFilesRemoved",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Triggers when a file is removed."
      },
      "complexType": {
        "original": "File[]",
        "resolved": "File[]",
        "references": {
          "File": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balClick",
      "name": "balClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input has clicked."
      },
      "complexType": {
        "original": "MouseEvent",
        "resolved": "MouseEvent",
        "references": {
          "MouseEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balBlur",
      "name": "balBlur",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input loses focus."
      },
      "complexType": {
        "original": "FocusEvent",
        "resolved": "FocusEvent",
        "references": {
          "FocusEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balFocus",
      "name": "balFocus",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input has focus."
      },
      "complexType": {
        "original": "FocusEvent",
        "resolved": "FocusEvent",
        "references": {
          "FocusEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balRejectedFileEventEmitter",
      "name": "balRejectedFile",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Triggers when a file is rejected due to not allowed MIME-Type and so on."
      },
      "complexType": {
        "original": "FileUploadRejectedFile",
        "resolved": "FileUploadRejectedFile",
        "references": {
          "FileUploadRejectedFile": {
            "location": "import",
            "path": "./bal-file-upload.type"
          }
        }
      }
    }]; }
  static get methods() { return {
    "clear": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets the file list to an empty list",
        "tags": []
      }
    },
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets focus on the native `input`. Use this method instead of the global\n`input.focus()`.",
        "tags": []
      }
    },
    "setBlur": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets blur on the native `input`. Use this method instead of the global\n`input.blur()`.",
        "tags": [{
            "name": "internal",
            "text": undefined
          }]
      }
    },
    "getInputElement": {
      "complexType": {
        "signature": "() => Promise<HTMLInputElement>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLInputElement>"
      },
      "docs": {
        "text": "Returns the native `<input>` element used under the hood.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "value",
      "methodName": "onValueChange"
    }]; }
  static get listeners() { return [{
      "name": "dragenter",
      "method": "dragenterHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "dragover",
      "method": "dragoverHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "dragleave",
      "method": "dragleaveHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "drop",
      "method": "dropHandler",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
let UploadIds = 0;
