import { Component, h, Host, Element, Prop, Watch, Event, Listen, Method, State, } from '@stencil/core';
import Big from 'big.js';
import { formatLocaleNumber } from '@baloise/web-app-utils';
import { debounceEvent, findItemLabel, inheritAttributes } from '../../../helpers/helpers';
import { inputListenOnClick } from '../../../helpers/form-input.helpers';
import { attachComponentToConfig, defaultConfig, detachComponentToConfig, } from '../../../config';
import { BEM } from '../../../utils/bem';
export class InputStepper {
  constructor() {
    this.inputId = `bal-input-stepper${InputStepperIds++}`;
    this.inheritedAttributes = {};
    this.hasFocus = false;
    this.language = defaultConfig.language;
    this.region = defaultConfig.region;
    /**
     * The name of the control, which is submitted with the form data.
     */
    this.name = this.inputId;
    /**
     * The min value the input can have
     */
    this.min = 0;
    /**
     * The max value the input can have
     */
    this.max = 10;
    /**
     * The steps in which the input increases or decreases
     */
    this.steps = 1;
    /**
     * If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants.
     */
    this.disabled = false;
    /**
     * If `true` the element can not mutated, meaning the user can not edit the control.
     */
    this.readonly = false;
    /**
     * If `true` the input is shown as invalid
     */
    this.invalid = false;
    /**
     * Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`.
     */
    this.debounce = 0;
    /**
     * The value of the input. Only allows values in the range of the min max attribute.
     */
    this.value = 0;
    this.initialValue = 0;
  }
  debounceChanged() {
    this.balChange = debounceEvent(this.balChange, this.debounce);
  }
  listenOnClick(event) {
    inputListenOnClick(this, event);
  }
  resetHandler(event) {
    const formElement = event.target;
    if (formElement === null || formElement === void 0 ? void 0 : formElement.contains(this.el)) {
      this.value = this.initialValue;
    }
  }
  connectedCallback() {
    this.debounceChanged();
    attachComponentToConfig(this);
    this.initialValue = this.value;
  }
  componentWillLoad() {
    this.inheritedAttributes = inheritAttributes(this.el, ['aria-label', 'tabindex', 'title']);
  }
  disconnectedCallback() {
    detachComponentToConfig(this);
  }
  configChanged(state) {
    this.language = state.language;
    this.region = state.region;
  }
  /**
   * Returns the native `<input>` element used under the hood.
   */
  getInputElement() {
    return Promise.resolve(this.nativeInput);
  }
  increase() {
    if (!this.disabled && !this.readonly) {
      const newValue = new Big(this.value).plus(this.steps).toNumber();
      if (newValue <= this.max) {
        this.value = newValue;
        this.balInput.emit(newValue);
        this.balChange.emit(newValue);
        this.balIncrease.emit(newValue);
      }
    }
  }
  decrease() {
    if (!this.disabled && !this.readonly) {
      const newValue = new Big(this.value).minus(this.steps).toNumber();
      if (newValue >= this.min) {
        this.value = newValue;
        this.balInput.emit(newValue);
        this.balChange.emit(newValue);
        this.balDecrease.emit(newValue);
      }
    }
  }
  render() {
    const labelId = this.inputId + '-lbl';
    const label = findItemLabel(this.el);
    if (label) {
      label.id = labelId;
      label.htmlFor = this.inputId;
    }
    const block = BEM.block('input-stepper');
    const elInput = block.element('input');
    const elInner = block.element('inner');
    const elText = elInner.element('text');
    return (h(Host, { "aria-disabled": this.disabled ? 'true' : null, "aria-focused": this.hasFocus ? 'true' : null, class: Object.assign({}, block.class()) },
      h("div", { class: Object.assign({ 'is-flex': true, 'fg-1': true, 'is-justify-content-center': true, 'is-align-items-center': true }, elInner.class()) },
        h("bal-button", { size: "small", square: true, "data-testid": "decrease", outlined: !this.invalid, icon: "minus", color: this.invalid ? 'danger' : 'info', disabled: this.disabled || this.readonly || this.value <= this.min, onClick: _ => this.decrease() }),
        h("bal-text", { space: "none", color: this.disabled || this.readonly ? 'grey' : this.invalid ? 'danger' : '', bold: true, class: Object.assign({}, elText.class()) }, formatLocaleNumber(`${this.language}-${this.region}`, this.value)),
        h("bal-button", { size: "small", "data-testid": "increase", square: true, outlined: !this.invalid, icon: "plus", color: this.invalid ? 'danger' : 'info', disabled: this.disabled || this.readonly || this.value >= this.max, onClick: _ => this.increase() })),
      h("input", Object.assign({ class: Object.assign({}, elInput.class()), type: "text", value: this.value, name: this.name, ref: inputEl => (this.nativeInput = inputEl), id: this.inputId, "aria-labelledby": labelId, readonly: this.readonly, disabled: this.disabled }, this.inheritedAttributes))));
  }
  static get is() { return "bal-input-stepper"; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the control, which is submitted with the form data."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "this.inputId"
    },
    "min": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The min value the input can have"
      },
      "attribute": "min",
      "reflect": false,
      "defaultValue": "0"
    },
    "max": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The max value the input can have"
      },
      "attribute": "max",
      "reflect": false,
      "defaultValue": "10"
    },
    "steps": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The steps in which the input increases or decreases"
      },
      "attribute": "steps",
      "reflect": false,
      "defaultValue": "1"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the element can not mutated, meaning the user can not edit the control."
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "false"
    },
    "invalid": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the input is shown as invalid"
      },
      "attribute": "invalid",
      "reflect": false,
      "defaultValue": "false"
    },
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`."
      },
      "attribute": "debounce",
      "reflect": false,
      "defaultValue": "0"
    },
    "value": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value of the input. Only allows values in the range of the min max attribute."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "0"
    }
  }; }
  static get states() { return {
    "hasFocus": {},
    "language": {},
    "region": {}
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input value has changed."
      },
      "complexType": {
        "original": "Events.BalInputStepperChangeDetail",
        "resolved": "number | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }, {
      "method": "balInput",
      "name": "balInput",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input value has changed."
      },
      "complexType": {
        "original": "Events.BalInputStepperInputDetail",
        "resolved": "number | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }, {
      "method": "balIncrease",
      "name": "balIncrease",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input value has increased."
      },
      "complexType": {
        "original": "Events.BalInputStepperChangeDetail",
        "resolved": "number | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }, {
      "method": "balDecrease",
      "name": "balDecrease",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input value has decreased."
      },
      "complexType": {
        "original": "Events.BalInputStepperChangeDetail",
        "resolved": "number | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }]; }
  static get methods() { return {
    "getInputElement": {
      "complexType": {
        "signature": "() => Promise<HTMLInputElement | undefined>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLInputElement | undefined>"
      },
      "docs": {
        "text": "Returns the native `<input>` element used under the hood.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "debounce",
      "methodName": "debounceChanged"
    }]; }
  static get listeners() { return [{
      "name": "click",
      "method": "listenOnClick",
      "target": "document",
      "capture": true,
      "passive": false
    }, {
      "name": "reset",
      "method": "resetHandler",
      "target": "document",
      "capture": true,
      "passive": false
    }]; }
}
let InputStepperIds = 0;
