import { Component, Element, Event, h, Host, Listen, Method, Prop, State, Watch, } from '@stencil/core';
import { debounceEvent, findItemLabel, inheritAttributes } from '../../../helpers/helpers';
import { getInputTarget, inputHandleBlur, inputHandleChange, inputHandleClick, inputHandleFocus, inputHandleHostClick, inputHandleReset, inputListenOnClick, inputSetBlur, inputSetFocus, stopEventBubbling, } from '../../../helpers/form-input.helpers';
import { formatClaim, formatOffer, formatPolicy, MAX_LENGTH_CLAIM_NUMBER, MAX_LENGTH_CONTRACT_NUMBER, MAX_LENGTH_OFFER_NUMBER, } from './bal-input-util';
import isNil from 'lodash.isnil';
import { ACTION_KEYS, isCtrlOrCommandKey, NUMBER_KEYS } from '../../../constants/keys.constant';
import { BEM } from '../../../utils/bem';
export class Input {
  constructor() {
    this.inputId = `bal-input-${InputIds++}`;
    this.inheritedAttributes = {};
    this.inputValue = this.value;
    this.initialValue = this.value || '';
    this.hasFocus = false;
    /**
     * The name of the control, which is submitted with the form data.
     */
    this.name = this.inputId;
    /**
     * If `true` the component gets a invalid style.
     */
    this.invalid = false;
    /**
     * Defines the text align of the input value.
     */
    this.textAlign = 'left';
    /**
     * Defines the type of the input (text, number, email ...).
     */
    this.type = 'text';
    /**
     * Indicates whether and how the text value should be automatically capitalized as it is entered/edited by the user.
     * Available options: `"off"`, `"none"`, `"on"`, `"sentences"`, `"words"`, `"characters"`.
     */
    this.autocapitalize = 'off';
    /**
     * Indicates whether the value of the control can be automatically completed by the browser.
     */
    this.autocomplete = 'off';
    /**
     * Whether auto correction should be enabled when the user is entering/editing the text value.
     */
    this.autocorrect = 'off';
    /**
     * This Boolean attribute lets you specify that a form control should have input focus when the page loads.
     */
    this.autofocus = false;
    /**
     * Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`.
     */
    this.debounce = 0;
    /**
     * If `true`, the user must fill in a value before submitting a form.
     */
    this.required = false;
    /**
     * If `true`, the element will have its spelling and grammar checked.
     */
    this.spellcheck = false;
    /**
     * If `true` this component can be placed on dark background
     */
    this.inverted = false;
    /**
     * If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants.
     */
    this.disabled = false;
    /**
     * If `true` the element can not mutated, meaning the user can not edit the control.
     */
    this.readonly = false;
    /**
     * If `true` the input gets a clickable cursor style
     */
    this.clickable = false;
    /**
     * @deprecated
     * If `true` on mobile device the number keypad is active. Use the <bal-number-input> component instead.
     */
    this.numberInput = false;
    /**
     * @internal
     * If `true` the input will get some right padding.
     */
    this.hasIconRight = false;
    /**
     * The value of the input.
     */
    this.value = undefined;
    /**
     * Mask of the input field. It defines what the user can enter and how the format looks like. Currently, only for Switzerland formatted.
     * Formatting for 'contract-number': '99/1.234.567-1'
     * Formatting for 'claim-number': ('73/001217/16.9')
     * Formatting for 'offer-number': ('98/7.654.321')
     */
    this.mask = undefined;
    this.onInput = (ev) => {
      var _a, _b, _c;
      const input = getInputTarget(ev);
      const cursorPositionStart = (_a = ev.target) === null || _a === void 0 ? void 0 : _a.selectionStart;
      const cursorPositionEnd = (_b = ev.target) === null || _b === void 0 ? void 0 : _b.selectionEnd;
      if (this.allowedKeyPress && input && !this.mask) {
        const regex = new RegExp('^' + this.allowedKeyPress + '$');
        this.inputValue = input.value = input.value
          .split('')
          .filter(val => regex.test(val))
          .join('');
      }
      if (input) {
        if (input.value) {
          switch (this.mask) {
            case 'contract-number': {
              this.inputValue = input.value.replace(/\D/g, '');
              if (this.inputValue.length > MAX_LENGTH_CONTRACT_NUMBER) {
                this.inputValue = this.inputValue.substring(0, MAX_LENGTH_CONTRACT_NUMBER);
              }
              input.value = formatPolicy(this.inputValue);
              if (cursorPositionStart < this.inputValue.length) {
                input.setSelectionRange(cursorPositionStart, cursorPositionEnd);
              }
              break;
            }
            case 'offer-number': {
              this.inputValue = input.value.replace(/\D/g, '');
              if (this.inputValue.length > MAX_LENGTH_OFFER_NUMBER) {
                this.inputValue = this.inputValue.substring(0, MAX_LENGTH_OFFER_NUMBER);
              }
              input.value = formatOffer(this.inputValue);
              if (cursorPositionStart < this.inputValue.length) {
                input.setSelectionRange(cursorPositionStart, cursorPositionEnd);
              }
              break;
            }
            case 'claim-number': {
              this.inputValue = input.value.replace(/[^\dX]/g, '');
              const inputParts = [
                this.inputValue.substring(0, MAX_LENGTH_CLAIM_NUMBER - 1),
                this.inputValue.substring(MAX_LENGTH_CLAIM_NUMBER - 1, MAX_LENGTH_CLAIM_NUMBER),
                this.inputValue.substring(MAX_LENGTH_CLAIM_NUMBER),
              ].filter(val => val.length > 0);
              switch (inputParts.length) {
                case 1:
                  this.inputValue = `${inputParts[0].replace(/\D/g, '')}`;
                  break;
                case 2:
                  this.inputValue = `${inputParts[0].replace(/\D/g, '')}${inputParts[1]}`;
                  break;
                default:
                  this.inputValue = `${inputParts[0].replace(/\D/g, '')}${inputParts[1]}${(_c = inputParts[2]) === null || _c === void 0 ? void 0 : _c.replace(/\D/g, '')}`;
              }
              if (this.inputValue.length > MAX_LENGTH_CLAIM_NUMBER) {
                this.inputValue = this.inputValue.substring(0, MAX_LENGTH_CLAIM_NUMBER);
              }
              input.value = formatClaim(this.inputValue);
              if (cursorPositionStart < this.inputValue.length) {
                input.setSelectionRange(cursorPositionStart, cursorPositionEnd);
              }
              break;
            }
            default:
              this.inputValue = input.value;
          }
        }
        else {
          this.inputValue = input.value;
        }
      }
      this.balInput.emit(this.inputValue);
    };
    this.onFocus = (event) => inputHandleFocus(this, event);
    this.onBlur = (ev) => {
      inputHandleBlur(this, ev);
      const input = ev.target;
      if (input) {
        if (this.mask === undefined) {
          input.value = this.getFormattedValue();
        }
        inputHandleChange(this);
      }
    };
    this.onKeydown = (event) => {
      if (this.mask !== undefined && !isNil(event) && !isCtrlOrCommandKey(event)) {
        if (!(this.getMaskAllowedKeys().includes(event.key) ||
          (this.mask === 'claim-number' &&
            event.key === 'X' &&
            this.inputValue &&
            this.inputValue.length >= MAX_LENGTH_CLAIM_NUMBER - 1))) {
          // do not trigger next event -> on input
          return stopEventBubbling(event);
        }
      }
      if (this.allowedKeyPress && !this.mask && !isNil(event) && !isCtrlOrCommandKey(event)) {
        const regex = new RegExp('^' + this.allowedKeyPress + '$');
        if (!regex.test(event.key) && ![...ACTION_KEYS].includes(event.key)) {
          return stopEventBubbling(event);
        }
      }
    };
    this.onClick = (event) => inputHandleClick(this, event);
    this.handleClick = (event) => inputHandleHostClick(this, event);
  }
  debounceChanged() {
    this.balChange = debounceEvent(this.balChange, this.debounce);
  }
  numberInputHandler() {
    if (this.numberInput) {
      console.warn('[DEPRECATED] - Please use the component <bal-number-input> instead');
    }
  }
  decimalHandler() {
    if (this.decimal) {
      console.warn('[DEPRECATED] - Please use the component <bal-number-input> instead');
    }
  }
  listenOnClick(event) {
    inputListenOnClick(this, event);
  }
  resetHandler(event) {
    const formElement = event.target;
    if (formElement === null || formElement === void 0 ? void 0 : formElement.contains(this.el)) {
      inputHandleReset(this, this.initialValue, this.resetHandlerTimer);
    }
  }
  connectedCallback() {
    this.debounceChanged();
    this.initialValue = this.value || '';
  }
  componentWillLoad() {
    this.inheritedAttributes = inheritAttributes(this.el, ['aria-label', 'tabindex', 'title']);
  }
  componentDidLoad() {
    this.inputValue = this.value;
  }
  /**
   * Sets focus on the native `input` in `bal-input`. Use this method instead of the global
   * `input.focus()`.
   */
  async setFocus() {
    inputSetFocus(this);
  }
  /**
   * Sets blur on the native `input` in `bal-input`. Use this method instead of the global
   * `input.blur()`.
   * @internal
   */
  async setBlur() {
    inputSetBlur(this);
  }
  /**
   * Returns the native `<input>` element used under the hood.
   */
  getInputElement() {
    return Promise.resolve(this.nativeInput);
  }
  getRawValue() {
    const value = (this.value || '').toString();
    return value;
  }
  getFormattedValue() {
    const value = this.getRawValue();
    const suffix = this.suffix !== undefined && value !== undefined && value !== '' ? ' ' + this.suffix : '';
    return `${value}${suffix}`;
  }
  getMaskAllowedKeys() {
    return [...NUMBER_KEYS, ...ACTION_KEYS];
  }
  render() {
    /**
     * @deprecated should be removed in the next breaking change release.
     */
    if (this.numberInput) {
      const numberValue = !this.value ? undefined : parseFloat(this.value);
      return (h("bal-number-input", { value: numberValue, name: this.name, disabled: this.disabled, readonly: this.readonly, invalid: this.invalid, decimal: this.decimal, suffix: this.suffix, placeholder: this.placeholder, required: this.required, debounce: this.debounce, onBalFocus: e => this.balFocus.emit(e.detail), onBalBlur: e => this.balBlur.emit(e.detail), onBalInput: e => { var _a; return this.balInput.emit((_a = e.detail) === null || _a === void 0 ? void 0 : _a.toString()); }, onBalChange: e => { var _a; return this.balChange.emit((_a = e.detail) === null || _a === void 0 ? void 0 : _a.toString()); }, onBalKeyPress: e => this.balKeyPress.emit(e.detail), onBalClick: e => this.balClick.emit(e.detail) }));
    }
    let value = this.hasFocus ? this.getRawValue() : this.getFormattedValue();
    if (this.mask !== undefined) {
      switch (this.mask) {
        case 'contract-number':
          value = formatPolicy(value);
          break;
        case 'claim-number':
          value = formatClaim(value);
          break;
        case 'offer-number':
          value = formatOffer(value);
      }
    }
    const labelId = this.inputId + '-lbl';
    const label = findItemLabel(this.el);
    if (label) {
      label.id = labelId;
      label.htmlFor = this.inputId;
    }
    let inputProps = {};
    if (this.pattern) {
      inputProps = { pattern: this.pattern };
    }
    const block = BEM.block('input');
    return (h(Host, { onClick: this.handleClick, "aria-disabled": this.disabled ? 'true' : null, class: Object.assign({}, block.class()) },
      h("input", Object.assign({ class: {
          'input': true,
          'data-test-input': true,
          'is-disabled': this.disabled || this.readonly,
          'is-danger': this.invalid,
          'is-inverted': this.inverted,
          'clickable': this.clickable,
          'has-icon-right': this.hasIconRight,
          'has-text-centered': this.textAlign === 'center',
          'has-text-right': this.textAlign === 'right',
        }, ref: inputEl => (this.nativeInput = inputEl), id: this.inputId, "aria-labelledby": labelId, disabled: this.disabled, accept: this.accept, inputMode: this.inputmode, autoCapitalize: this.autocapitalize, autocomplete: this.autocomplete, autocorrect: this.autocorrect, autofocus: this.autofocus, min: this.min, max: this.max, minLength: this.minLength, maxLength: this.maxLength, multiple: this.multiple, name: this.name, placeholder: this.placeholder || '', readonly: this.readonly, required: this.required, spellcheck: this.spellcheck, type: this.type, value: value, onFocus: this.onFocus, onInput: ev => this.onInput(ev), onKeyDown: e => this.onKeydown(e), onBlur: this.onBlur, onClick: this.onClick, onKeyPress: e => this.balKeyPress.emit(e) }, inputProps, this.inheritedAttributes))));
  }
  static get is() { return "bal-input"; }
  static get properties() { return {
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the control, which is submitted with the form data."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "this.inputId"
    },
    "invalid": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the component gets a invalid style."
      },
      "attribute": "invalid",
      "reflect": false,
      "defaultValue": "false"
    },
    "textAlign": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'center' | 'left' | 'right'",
        "resolved": "\"center\" | \"left\" | \"right\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the text align of the input value."
      },
      "attribute": "text-align",
      "reflect": false,
      "defaultValue": "'left'"
    },
    "type": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalInputInputType",
        "resolved": "\"button\" | \"checkbox\" | \"color\" | \"date\" | \"datetime-local\" | \"email\" | \"file\" | \"image\" | \"month\" | \"number\" | \"password\" | \"radio\" | \"range\" | \"search\" | \"tel\" | \"text\" | \"time\" | \"url\" | \"week\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the type of the input (text, number, email ...)."
      },
      "attribute": "type",
      "reflect": false,
      "defaultValue": "'text'"
    },
    "accept": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "If the value of the type attribute is `\"file\"`, then this attribute will indicate the types of files that the server accepts, otherwise it will be ignored. The value must be a comma-separated list of unique content type specifiers."
      },
      "attribute": "accept",
      "reflect": false
    },
    "autocapitalize": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates whether and how the text value should be automatically capitalized as it is entered/edited by the user.\nAvailable options: `\"off\"`, `\"none\"`, `\"on\"`, `\"sentences\"`, `\"words\"`, `\"characters\"`."
      },
      "attribute": "autocapitalize",
      "reflect": false,
      "defaultValue": "'off'"
    },
    "autocomplete": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalInputAutocomplete",
        "resolved": "\"on\" | \"off\" | \"name\" | \"honorific-prefix\" | \"given-name\" | \"additional-name\" | \"family-name\" | \"honorific-suffix\" | \"nickname\" | \"email\" | \"username\" | \"new-password\" | \"current-password\" | \"one-time-code\" | \"organization-title\" | \"organization\" | \"street-address\" | \"address-line1\" | \"address-line2\" | \"address-line3\" | \"address-level4\" | \"address-level3\" | \"address-level2\" | \"address-level1\" | \"country\" | \"country-name\" | \"postal-code\" | \"cc-name\" | \"cc-given-name\" | \"cc-additional-name\" | \"cc-family-name\" | \"cc-number\" | \"cc-exp\" | \"cc-exp-month\" | \"cc-exp-year\" | \"cc-csc\" | \"cc-type\" | \"transaction-currency\" | \"transaction-amount\" | \"language\" | \"bday\" | \"bday-day\" | \"bday-month\" | \"bday-year\" | \"sex\" | \"tel\" | \"tel-country-code\" | \"tel-national\" | \"tel-area-code\" | \"tel-local\" | \"tel-extension\" | \"impp\" | \"url\" | \"photo\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Indicates whether the value of the control can be automatically completed by the browser."
      },
      "attribute": "autocomplete",
      "reflect": false,
      "defaultValue": "'off'"
    },
    "autocorrect": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalInputAutocorrect",
        "resolved": "\"off\" | \"on\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Whether auto correction should be enabled when the user is entering/editing the text value."
      },
      "attribute": "autocorrect",
      "reflect": false,
      "defaultValue": "'off'"
    },
    "autofocus": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "This Boolean attribute lets you specify that a form control should have input focus when the page loads."
      },
      "attribute": "autofocus",
      "reflect": false,
      "defaultValue": "false"
    },
    "debounce": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`."
      },
      "attribute": "debounce",
      "reflect": false,
      "defaultValue": "0"
    },
    "placeholder": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Instructional text that shows before the input has a value."
      },
      "attribute": "placeholder",
      "reflect": false
    },
    "max": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The maximum value, which must not be less than its minimum (min attribute) value."
      },
      "attribute": "max",
      "reflect": false
    },
    "maxLength": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Defines the max length of the value."
      },
      "attribute": "max-length",
      "reflect": false
    },
    "min": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The minimum value, which must not be greater than its maximum (max attribute) value."
      },
      "attribute": "min",
      "reflect": false
    },
    "minLength": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Defines the min length of the value."
      },
      "attribute": "min-length",
      "reflect": false
    },
    "multiple": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "If `true`, the user can enter more than one value. This attribute applies when the type attribute is set to `\"email\"` or `\"file\"`, otherwise it is ignored."
      },
      "attribute": "multiple",
      "reflect": false
    },
    "pattern": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "A regular expression that the value is checked against. The pattern must match the entire value, not just some subset. Use the title attribute to describe the pattern to help the user. This attribute applies when the value of the type attribute is `\"text\"`, `\"search\"`, `\"tel\"`, `\"url\"`, `\"email\"`, `\"date\"`, or `\"password\"`, otherwise it is ignored. When the type attribute is `\"date\"`, `pattern` will only be used in browsers that do not support the `\"date\"` input type natively. See https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/date for more information."
      },
      "attribute": "pattern",
      "reflect": false
    },
    "allowedKeyPress": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "A regular expression that the key of the key press event is checked against and if not matching the expression the event will be prevented."
      },
      "attribute": "allowed-key-press",
      "reflect": false
    },
    "required": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the user must fill in a value before submitting a form."
      },
      "attribute": "required",
      "reflect": false,
      "defaultValue": "false"
    },
    "spellcheck": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the element will have its spelling and grammar checked."
      },
      "attribute": "spellcheck",
      "reflect": false,
      "defaultValue": "false"
    },
    "inverted": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` this component can be placed on dark background"
      },
      "attribute": "inverted",
      "reflect": false,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "false"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the element can not mutated, meaning the user can not edit the control."
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "false"
    },
    "clickable": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true` the input gets a clickable cursor style"
      },
      "attribute": "clickable",
      "reflect": false,
      "defaultValue": "false"
    },
    "numberInput": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "If `true` on mobile device the number keypad is active. Use the <bal-number-input> component instead."
          }],
        "text": ""
      },
      "attribute": "number-input",
      "reflect": false,
      "defaultValue": "false"
    },
    "decimal": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "deprecated",
            "text": "Defines the allowed decimal points for the `number-input`. Use the <bal-number-input> component instead."
          }],
        "text": ""
      },
      "attribute": "decimal",
      "reflect": false
    },
    "suffix": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Adds a suffix the the input-value after blur."
      },
      "attribute": "suffix",
      "reflect": false
    },
    "hasIconRight": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": "If `true` the input will get some right padding."
          }],
        "text": ""
      },
      "attribute": "has-icon-right",
      "reflect": false,
      "defaultValue": "false"
    },
    "inputmode": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalInputInputMode",
        "resolved": "\"decimal\" | \"email\" | \"none\" | \"numeric\" | \"search\" | \"tel\" | \"text\" | \"url\" | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "A hint to the browser for which keyboard to display.\nPossible values: `\"none\"`, `\"text\"`, `\"tel\"`, `\"url\"`,\n`\"email\"`, `\"numeric\"`, `\"decimal\"`, and `\"search\"`."
      },
      "attribute": "inputmode",
      "reflect": false
    },
    "value": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The value of the input."
      },
      "attribute": "value",
      "reflect": true,
      "defaultValue": "undefined"
    },
    "mask": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalInputMask",
        "resolved": "\"claim-number\" | \"contract-number\" | \"offer-number\" | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Mask of the input field. It defines what the user can enter and how the format looks like. Currently, only for Switzerland formatted.\nFormatting for 'contract-number': '99/1.234.567-1'\nFormatting for 'claim-number': ('73/001217/16.9')\nFormatting for 'offer-number': ('98/7.654.321')"
      },
      "attribute": "mask",
      "reflect": false,
      "defaultValue": "undefined"
    }
  }; }
  static get states() { return {
    "hasFocus": {}
  }; }
  static get events() { return [{
      "method": "balInput",
      "name": "balInput",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a keyboard input occurred."
      },
      "complexType": {
        "original": "Events.BalInputInputDetail",
        "resolved": "string | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }, {
      "method": "balBlur",
      "name": "balBlur",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a keyboard input occurred."
      },
      "complexType": {
        "original": "FocusEvent",
        "resolved": "FocusEvent",
        "references": {
          "FocusEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balClick",
      "name": "balClick",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input has clicked."
      },
      "complexType": {
        "original": "MouseEvent",
        "resolved": "MouseEvent",
        "references": {
          "MouseEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balKeyPress",
      "name": "balKeyPress",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when a keyboard key has pressed."
      },
      "complexType": {
        "original": "KeyboardEvent",
        "resolved": "KeyboardEvent",
        "references": {
          "KeyboardEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balFocus",
      "name": "balFocus",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input has focus."
      },
      "complexType": {
        "original": "FocusEvent",
        "resolved": "FocusEvent",
        "references": {
          "FocusEvent": {
            "location": "global"
          }
        }
      }
    }, {
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the input value has changed."
      },
      "complexType": {
        "original": "Events.BalInputChangeDetail",
        "resolved": "string | undefined",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../types"
          }
        }
      }
    }]; }
  static get methods() { return {
    "setFocus": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets focus on the native `input` in `bal-input`. Use this method instead of the global\n`input.focus()`.",
        "tags": []
      }
    },
    "setBlur": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Sets blur on the native `input` in `bal-input`. Use this method instead of the global\n`input.blur()`.",
        "tags": [{
            "name": "internal",
            "text": undefined
          }]
      }
    },
    "getInputElement": {
      "complexType": {
        "signature": "() => Promise<HTMLInputElement>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLInputElement": {
            "location": "global"
          }
        },
        "return": "Promise<HTMLInputElement>"
      },
      "docs": {
        "text": "Returns the native `<input>` element used under the hood.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "debounce",
      "methodName": "debounceChanged"
    }, {
      "propName": "numberInput",
      "methodName": "numberInputHandler"
    }, {
      "propName": "decimal",
      "methodName": "decimalHandler"
    }]; }
  static get listeners() { return [{
      "name": "click",
      "method": "listenOnClick",
      "target": "document",
      "capture": true,
      "passive": false
    }, {
      "name": "reset",
      "method": "resetHandler",
      "target": "document",
      "capture": true,
      "passive": false
    }]; }
}
let InputIds = 0;
