import { Component, h, Host, Prop, Element, Event, Watch, Method, Listen, } from '@stencil/core';
import { stopEventBubbling } from '../../../../helpers/form-input.helpers';
import { findItemLabel, inheritAttributes, isDescendant } from '../../../../helpers/helpers';
import { BEM } from '../../../../utils/bem';
export class RadioGroup {
  constructor() {
    this.inputId = `bal-rg-${radioGroupIds++}`;
    this.inheritedAttributes = {};
    this.initialValue = '';
    /**
     * Defines the layout of the radio button
     */
    this.interface = undefined;
    /**
     * The name of the control, which is submitted with the form data.
     */
    this.name = this.inputId;
    /**
     * Displays the checkboxes vertically
     */
    this.vertical = false;
    /**
     * If `true`, the controls will be vertically on mobile devices.
     */
    this.verticalOnMobile = false;
    /**
     * Uses the whole width
     */
    this.expanded = false;
    /**
     * If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants.
     */
    this.disabled = undefined;
    /**
     * If `true` the element can not mutated, meaning the user can not edit the control.
     */
    this.readonly = undefined;
    /**
     * The value of the control.
     */
    this.value = '';
    this.onClick = (ev) => {
      const element = ev.target;
      if (element.href) {
        return;
      }
      ev.preventDefault();
      const selectedRadio = ev.target && ev.target.closest('bal-radio');
      if (selectedRadio) {
        if (selectedRadio.disabled || selectedRadio.readonly) {
          ev.stopPropagation();
          return;
        }
        const currentValue = this.value;
        const newValue = selectedRadio.value;
        if (newValue !== currentValue) {
          this.value = newValue;
          this.balChange.emit(this.value);
        }
      }
    };
  }
  disabledChanged(value) {
    if (value !== undefined) {
      this.children.forEach(child => {
        child.disabled = value;
      });
    }
  }
  readonlyChanged(value) {
    if (value !== undefined) {
      this.children.forEach(child => {
        child.readonly = value;
      });
    }
  }
  valueChanged(value, oldValue) {
    if (value !== oldValue) {
      this.sync();
    }
  }
  listenOnClick(ev) {
    if (isDescendant(this.el, ev.target)) {
      stopEventBubbling(ev);
    }
  }
  resetHandler(event) {
    const formElement = event.target;
    if (formElement === null || formElement === void 0 ? void 0 : formElement.contains(this.el)) {
      this.value = this.initialValue;
      this.sync();
    }
  }
  connectedCallback() {
    this.initialValue = this.value;
  }
  componentWillLoad() {
    this.inheritedAttributes = inheritAttributes(this.el, ['aria-label', 'tabindex', 'title']);
    this.sync();
    this.disabledChanged(this.disabled);
    this.readonlyChanged(this.readonly);
  }
  get children() {
    return Array.from(this.el.querySelectorAll('bal-radio'));
  }
  /** @internal */
  async setValue(value) {
    this.value = value;
  }
  sync() {
    this.children.forEach((radio) => {
      if (this.interface) {
        radio.interface = this.interface;
      }
      radio.checked = radio.value === this.value;
    });
  }
  render() {
    const label = findItemLabel(this.el);
    const block = BEM.block('radio-checkbox-group');
    const innerEl = block.element('inner');
    return (h(Host, Object.assign({ class: Object.assign({}, block.class()), role: "radiogroup", "aria-labelledby": label === null || label === void 0 ? void 0 : label.id, "aria-disabled": this.disabled ? 'true' : null, onClick: this.onClick }, this.inheritedAttributes),
      h("div", { class: Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({}, innerEl.class()), innerEl.modifier('vertical-mobile').class(this.verticalOnMobile)), innerEl.modifier('vertical').class(this.vertical)), innerEl.modifier('expanded').class(this.expanded)), innerEl.modifier('select-button').class(this.interface === 'select-button')) },
        h("slot", null))));
  }
  static get is() { return "bal-radio-group"; }
  static get properties() { return {
    "interface": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalRadioGroupInterface",
        "resolved": "\"radio\" | \"select-button\" | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Defines the layout of the radio button"
      },
      "attribute": "interface",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "name": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The name of the control, which is submitted with the form data."
      },
      "attribute": "name",
      "reflect": false,
      "defaultValue": "this.inputId"
    },
    "vertical": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Displays the checkboxes vertically"
      },
      "attribute": "vertical",
      "reflect": false,
      "defaultValue": "false"
    },
    "verticalOnMobile": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the controls will be vertically on mobile devices."
      },
      "attribute": "vertical-on-mobile",
      "reflect": false,
      "defaultValue": "false"
    },
    "expanded": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Uses the whole width"
      },
      "attribute": "expanded",
      "reflect": false,
      "defaultValue": "false"
    },
    "disabled": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants."
      },
      "attribute": "disabled",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "readonly": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "If `true` the element can not mutated, meaning the user can not edit the control."
      },
      "attribute": "readonly",
      "reflect": false,
      "defaultValue": "undefined"
    },
    "value": {
      "type": "any",
      "mutable": true,
      "complexType": {
        "original": "number | string | boolean",
        "resolved": "boolean | number | string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The value of the control."
      },
      "attribute": "value",
      "reflect": false,
      "defaultValue": "''"
    }
  }; }
  static get events() { return [{
      "method": "balChange",
      "name": "balChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted when the checked property has changed."
      },
      "complexType": {
        "original": "Events.BalRadioGroupChangeDetail",
        "resolved": "boolean | number | string",
        "references": {
          "Events": {
            "location": "import",
            "path": "../../../../types"
          }
        }
      }
    }]; }
  static get methods() { return {
    "setValue": {
      "complexType": {
        "signature": "(value: number | string | boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": undefined
          }]
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "disabled",
      "methodName": "disabledChanged"
    }, {
      "propName": "readonly",
      "methodName": "readonlyChanged"
    }, {
      "propName": "value",
      "methodName": "valueChanged"
    }]; }
  static get listeners() { return [{
      "name": "balChange",
      "method": "listenOnClick",
      "target": "document",
      "capture": true,
      "passive": false
    }, {
      "name": "reset",
      "method": "resetHandler",
      "target": "document",
      "capture": true,
      "passive": false
    }]; }
}
let radioGroupIds = 0;
