import { Component, Host, h, State, Method, Listen, Prop, Event, Element, writeTask } from '@stencil/core';
import { dismiss, eventMethod, prepareOverlay } from '../../../helpers/overlays';
import { attachComponent, detachComponent } from '../../../helpers/framework-delegate';
import { deepReady, wait } from '../../../helpers/helpers';
import { getClassMap } from '../../../helpers/theme';
import { BodyScrollBlocker } from '../../../utils/toggle-scrolling-body';
export class Modal {
  constructor() {
    this.bodyScrollBlocker = BodyScrollBlocker();
    this.presented = false;
    /**
     * Defines the width of the modal body
     */
    this.modalWidth = 640;
    /**
     * Defines the space/padding of the modal
     */
    this.space = '';
    /**
     * If `true`, a backdrop will be displayed behind the modal.
     */
    this.hasBackdrop = true;
    /**
     * If `true`, the modal can be closed with the escape key or the little close button.
     */
    this.isClosable = true;
  }
  connectedCallback() {
    prepareOverlay(this);
  }
  /**
   * @internal Opens the modal.
   */
  async open() {
    await deepReady(this.usersElement);
    this.setModalActiveOnBody();
    writeTask(() => {
      if (this.modalBackgroundElement) {
        this.modalBackgroundElement.classList.add('fadeIn');
      }
      if (this.modalContentElement) {
        this.modalContentElement.classList.add('fadeInDown');
      }
    });
    if (this.presented) {
      return;
    }
    this.willPresent.emit();
    this.presented = true;
    await wait(150);
    writeTask(() => {
      if (this.modalBackgroundElement) {
        this.modalBackgroundElement.classList.remove('fadeIn');
      }
      if (this.modalContentElement) {
        this.modalContentElement.classList.remove('fadeInDown');
      }
    });
    this.didPresent.emit();
  }
  /**
   * @internal Closes the modal.
   */
  async close() {
    this.unsetModalActiveOnBody();
    this.presented = false;
  }
  /**
   * Presents the modal through the modal controller
   */
  async present() {
    this.setModalActiveOnBody();
    if (this.presented) {
      return;
    }
    const container = this.el.querySelector(`.bal-modal__content`);
    if (!container) {
      throw new Error('container is undefined');
    }
    const componentProps = Object.assign(Object.assign({}, this.componentProps), { modal: this.el });
    this.usersElement = await attachComponent(this.delegate, container, this.component, [], componentProps);
    await this.open();
  }
  /**
   * Closes the presented modal with the modal controller
   */
  async dismiss(data, role) {
    this.unsetModalActiveOnBody();
    if (this.delegate === undefined) {
      await this.close();
      return true;
    }
    this.willDismiss.emit({ data, role });
    const dismissed = await dismiss(this, data, role, async () => {
      writeTask(() => {
        if (this.modalBackgroundElement) {
          this.modalBackgroundElement.classList.add('fadeOut');
        }
        if (this.modalContentElement) {
          this.modalContentElement.classList.add('fadeOutUp');
        }
      });
      await wait(140);
    });
    if (dismissed) {
      await detachComponent(this.delegate, this.usersElement);
    }
    return dismissed;
  }
  /**
   * Returns a promise that resolves when the modal did dismiss.
   */
  onDidDismiss() {
    return eventMethod(this.el, 'balModalDidDismiss');
  }
  /**
   * Returns a promise that resolves when the modal will dismiss.
   */
  onWillDismiss() {
    return eventMethod(this.el, 'balModalWillDismiss');
  }
  async onClickCloseButton(event) {
    if (this.isClosable && this.presented && event && event.target) {
      const element = event.target;
      const closestBalButton = element.closest('bal-button');
      if (closestBalButton && closestBalButton.hasAttribute('modal-close')) {
        await this.dismiss(undefined, 'model-close');
      }
    }
  }
  async handleKeyUp(event) {
    const modals = Array.from(document.querySelectorAll('bal-modal')).filter(el => el.hasAttribute('aria-presented'));
    const numbers = modals
      .map(el => el.overlayIndex)
      .map(num => parseInt(num))
      .filter(num => !Number.isNaN(num));
    const max = Math.max(...numbers);
    if (this.overlayIndex === max) {
      event.preventDefault();
      event.stopPropagation();
      if (this.presented && this.isClosable) {
        if (event.key === 'Escape' || event.key === 'Esc') {
          event.preventDefault();
          if (this.delegate) {
            await this.dismiss(undefined, 'model-escape');
          }
          else {
            await this.close();
          }
        }
      }
    }
  }
  setModalActiveOnBody() {
    this.bodyScrollBlocker.block();
  }
  unsetModalActiveOnBody() {
    this.bodyScrollBlocker.allow();
  }
  render() {
    return (h(Host, { "aria-modal": "true", "aria-presented": this.presented ? 'true' : null, "data-testid": this.dataTestId, tabindex: "-1", class: Object.assign({ 'bal-modal': true, 'bal-modal--is-closable': this.isClosable, 'bal-modal--is-active': this.presented }, getClassMap(this.cssClass)), style: {
        '--bal-width': `${this.modalWidth}px`,
      } },
      h("div", { class: {
          'bal-modal__background': true,
          'is-hidden': !this.hasBackdrop,
        }, ref: div => (this.modalBackgroundElement = div) }),
      h("div", { class: {
          'bal-modal__container': true,
        }, ref: div => (this.modalContainerElement = div) },
        h("div", { class: {
            'bal-modal__content': true,
            [`bal-modal__content--has-spacing-${this.space}`]: true,
          }, ref: div => (this.modalContentElement = div) },
          h("slot", null)))));
  }
  static get is() { return "bal-modal"; }
  static get encapsulation() { return "scoped"; }
  static get properties() { return {
    "overlayIndex": {
      "type": "number",
      "mutable": true,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "attribute": "overlay-index",
      "reflect": false
    },
    "delegate": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Props.FrameworkDelegate",
        "resolved": "FrameworkDelegate | undefined",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      }
    },
    "dataTestId": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [{
            "name": "internal",
            "text": undefined
          }],
        "text": ""
      },
      "attribute": "data-test-id",
      "reflect": false
    },
    "modalWidth": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the width of the modal body"
      },
      "attribute": "modal-width",
      "reflect": false,
      "defaultValue": "640"
    },
    "space": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.BalModalSpace",
        "resolved": "\"\" | \"medium\" | \"small\"",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "Defines the space/padding of the modal"
      },
      "attribute": "space",
      "reflect": false,
      "defaultValue": "''"
    },
    "hasBackdrop": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, a backdrop will be displayed behind the modal."
      },
      "attribute": "has-backdrop",
      "reflect": false,
      "defaultValue": "true"
    },
    "isClosable": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "If `true`, the modal can be closed with the escape key or the little close button."
      },
      "attribute": "is-closable",
      "reflect": false,
      "defaultValue": "true"
    },
    "component": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "Props.ComponentRef",
        "resolved": "Function | HTMLElement | null | string",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": true,
      "optional": false,
      "docs": {
        "tags": [],
        "text": "The component to display inside of the modal."
      },
      "attribute": "component",
      "reflect": false
    },
    "componentProps": {
      "type": "unknown",
      "mutable": false,
      "complexType": {
        "original": "Props.ComponentProps",
        "resolved": "undefined | { [key: string]: any; }",
        "references": {
          "Props": {
            "location": "import",
            "path": "../../../types"
          }
        }
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "The data to pass to the modal component."
      }
    },
    "cssClass": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string | string[]",
        "resolved": "string | string[] | undefined",
        "references": {}
      },
      "required": false,
      "optional": true,
      "docs": {
        "tags": [],
        "text": "Additional classes to apply for custom CSS. If multiple classes are\nprovided they should be separated by spaces."
      },
      "attribute": "css-class",
      "reflect": false
    }
  }; }
  static get states() { return {
    "presented": {}
  }; }
  static get events() { return [{
      "method": "didPresent",
      "name": "balModalDidPresent",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the modal has presented."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "willPresent",
      "name": "balModalWillPresent",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted before the modal has presented."
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "willDismiss",
      "name": "balModalWillDismiss",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted before the modal has dismissed."
      },
      "complexType": {
        "original": "OverlayEventDetail",
        "resolved": "OverlayEventDetail<any>",
        "references": {
          "OverlayEventDetail": {
            "location": "import",
            "path": "./bal-modal.type"
          }
        }
      }
    }, {
      "method": "didDismiss",
      "name": "balModalDidDismiss",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": "Emitted after the modal has dismissed."
      },
      "complexType": {
        "original": "OverlayEventDetail",
        "resolved": "OverlayEventDetail<any>",
        "references": {
          "OverlayEventDetail": {
            "location": "import",
            "path": "./bal-modal.type"
          }
        }
      }
    }]; }
  static get methods() { return {
    "open": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": "Opens the modal."
          }]
      }
    },
    "close": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": [{
            "name": "internal",
            "text": "Closes the modal."
          }]
      }
    },
    "present": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "Presents the modal through the modal controller",
        "tags": []
      }
    },
    "dismiss": {
      "complexType": {
        "signature": "(data?: any, role?: string | undefined) => Promise<boolean>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "Closes the presented modal with the modal controller",
        "tags": []
      }
    },
    "onDidDismiss": {
      "complexType": {
        "signature": "<T = any>() => Promise<OverlayEventDetail<T>>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "OverlayEventDetail": {
            "location": "import",
            "path": "./bal-modal.type"
          },
          "T": {
            "location": "global"
          }
        },
        "return": "Promise<OverlayEventDetail<T>>"
      },
      "docs": {
        "text": "Returns a promise that resolves when the modal did dismiss.",
        "tags": []
      }
    },
    "onWillDismiss": {
      "complexType": {
        "signature": "<T = any>() => Promise<OverlayEventDetail<T>>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "OverlayEventDetail": {
            "location": "import",
            "path": "./bal-modal.type"
          },
          "T": {
            "location": "global"
          }
        },
        "return": "Promise<OverlayEventDetail<T>>"
      },
      "docs": {
        "text": "Returns a promise that resolves when the modal will dismiss.",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get listeners() { return [{
      "name": "click",
      "method": "onClickCloseButton",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "keyup",
      "method": "handleKeyUp",
      "target": "body",
      "capture": false,
      "passive": false
    }]; }
}
