import { ComponentInterface, EventEmitter } from '../../../stencil-public-runtime';
import { FormInput } from '../../../helpers/form-input.helpers';
import { Props, Events } from '../../../types';
export declare class Input implements ComponentInterface, FormInput<string | undefined> {
  private inputId;
  private inheritedAttributes;
  nativeInput?: HTMLInputElement;
  inputValue: string | undefined;
  initialValue: string;
  el: HTMLElement;
  hasFocus: boolean;
  /**
   * The name of the control, which is submitted with the form data.
   */
  name: string;
  /**
   * If `true` the component gets a invalid style.
   */
  invalid: boolean;
  /**
   * Defines the text align of the input value.
   */
  textAlign: 'center' | 'left' | 'right';
  /**
   * Defines the type of the input (text, number, email ...).
   */
  type: Props.BalInputInputType;
  /**
   * If the value of the type attribute is `"file"`, then this attribute will indicate the types of files that the server accepts, otherwise it will be ignored. The value must be a comma-separated list of unique content type specifiers.
   */
  accept?: string;
  /**
   * Indicates whether and how the text value should be automatically capitalized as it is entered/edited by the user.
   * Available options: `"off"`, `"none"`, `"on"`, `"sentences"`, `"words"`, `"characters"`.
   */
  autocapitalize: string;
  /**
   * Indicates whether the value of the control can be automatically completed by the browser.
   */
  autocomplete: Props.BalInputAutocomplete;
  /**
   * Whether auto correction should be enabled when the user is entering/editing the text value.
   */
  autocorrect: Props.BalInputAutocorrect;
  /**
   * This Boolean attribute lets you specify that a form control should have input focus when the page loads.
   */
  autofocus: boolean;
  /**
   * Set the amount of time, in milliseconds, to wait to trigger the `balChange` event after each keystroke. This also impacts form bindings such as `ngModel` or `v-model`.
   */
  debounce: number;
  protected debounceChanged(): void;
  /**
   * Instructional text that shows before the input has a value.
   */
  placeholder?: string;
  /**
   * The maximum value, which must not be less than its minimum (min attribute) value.
   */
  max?: string;
  /**
   * Defines the max length of the value.
   */
  maxLength?: number;
  /**
   * The minimum value, which must not be greater than its maximum (max attribute) value.
   */
  min?: string;
  /**
   * Defines the min length of the value.
   */
  minLength?: number;
  /**
   * If `true`, the user can enter more than one value. This attribute applies when the type attribute is set to `"email"` or `"file"`, otherwise it is ignored.
   */
  multiple?: boolean;
  /**
   * A regular expression that the value is checked against. The pattern must match the entire value, not just some subset. Use the title attribute to describe the pattern to help the user. This attribute applies when the value of the type attribute is `"text"`, `"search"`, `"tel"`, `"url"`, `"email"`, `"date"`, or `"password"`, otherwise it is ignored. When the type attribute is `"date"`, `pattern` will only be used in browsers that do not support the `"date"` input type natively. See https://developer.mozilla.org/en-US/docs/Web/HTML/Element/input/date for more information.
   */
  pattern?: string;
  /**
   * A regular expression that the key of the key press event is checked against and if not matching the expression the event will be prevented.
   */
  allowedKeyPress?: string;
  /**
   * If `true`, the user must fill in a value before submitting a form.
   */
  required: boolean;
  /**
   * If `true`, the element will have its spelling and grammar checked.
   */
  spellcheck: boolean;
  /**
   * If `true` this component can be placed on dark background
   */
  inverted: boolean;
  /**
   * If `true`, the element is not mutable, focusable, or even submitted with the form. The user can neither edit nor focus on the control, nor its form control descendants.
   */
  disabled: boolean;
  /**
   * If `true` the element can not mutated, meaning the user can not edit the control.
   */
  readonly: boolean;
  /**
   * If `true` the input gets a clickable cursor style
   */
  clickable: boolean;
  /**
   * @deprecated
   * If `true` on mobile device the number keypad is active. Use the <bal-number-input> component instead.
   */
  numberInput: boolean;
  numberInputHandler(): void;
  /**
   * @deprecated
   * Defines the allowed decimal points for the `number-input`. Use the <bal-number-input> component instead.
   */
  decimal?: number;
  decimalHandler(): void;
  /**
   * Adds a suffix the the input-value after blur.
   */
  suffix?: string;
  /**
   * @internal
   * If `true` the input will get some right padding.
   */
  hasIconRight: boolean;
  /**
   * A hint to the browser for which keyboard to display.
   * Possible values: `"none"`, `"text"`, `"tel"`, `"url"`,
   * `"email"`, `"numeric"`, `"decimal"`, and `"search"`.
   */
  inputmode?: Props.BalInputInputMode;
  /**
   * The value of the input.
   */
  value?: string;
  /**
   * Mask of the input field. It defines what the user can enter and how the format looks like. Currently, only for Switzerland formatted.
   * Formatting for 'contract-number': '99/1.234.567-1'
   * Formatting for 'claim-number': ('73/001217/16.9')
   * Formatting for 'offer-number': ('98/7.654.321')
   */
  mask?: Props.BalInputMask;
  /**
   * Emitted when a keyboard input occurred.
   */
  balInput: EventEmitter<Events.BalInputInputDetail>;
  /**
   * Emitted when a keyboard input occurred.
   */
  balBlur: EventEmitter<FocusEvent>;
  /**
   * Emitted when the input has clicked.
   */
  balClick: EventEmitter<MouseEvent>;
  /**
   * Emitted when a keyboard key has pressed.
   */
  balKeyPress: EventEmitter<KeyboardEvent>;
  /**
   * Emitted when the input has focus.
   */
  balFocus: EventEmitter<FocusEvent>;
  /**
   * Emitted when the input value has changed.
   */
  balChange: EventEmitter<Events.BalInputChangeDetail>;
  listenOnClick(event: UIEvent): void;
  private resetHandlerTimer?;
  resetHandler(event: UIEvent): void;
  connectedCallback(): void;
  componentWillLoad(): void;
  componentDidLoad(): void;
  /**
   * Sets focus on the native `input` in `bal-input`. Use this method instead of the global
   * `input.focus()`.
   */
  setFocus(): Promise<void>;
  /**
   * Sets blur on the native `input` in `bal-input`. Use this method instead of the global
   * `input.blur()`.
   * @internal
   */
  setBlur(): Promise<void>;
  /**
   * Returns the native `<input>` element used under the hood.
   */
  getInputElement(): Promise<HTMLInputElement>;
  private getRawValue;
  private getFormattedValue;
  private onInput;
  private onFocus;
  private onBlur;
  private getMaskAllowedKeys;
  private onKeydown;
  private onClick;
  private handleClick;
  render(): any;
}
